#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# Kolmogorov related functions
nValues = [
    2,
    5,
    10,
    20,
    50,
    100,
    200,
    500,
    1000,
    2000,
    5000,
    10000,
    20000,
    100000,
    1000000,
]
xValues = [
    0.0001,
    0.0002,
    0.0005,
    0.001,
    0.002,
    0.005,
    0.01,
    0.02,
    0.05,
    0.1,
    0.2,
    0.5,
    1.0,
    1.2,
    1.5,
    2.0,
    5.0,
    10.0,
    20.0,
    50.0,
]
for i in range(len(nValues)):
    n = nValues[i]
    for j in range(len(xValues)):
        x = xValues[j]
        cdf = ot.DistFunc.pKolmogorov(n, x)
        if cdf < 1e-16:
            cdf = 0.0
        ccdf = ot.DistFunc.pKolmogorov(n, x, True)
        if ccdf < 1e-16:
            ccdf = 0.0
        print(f"pKolmogorov( {n} ,  {x:.12g} )={cdf:.6g} , complementary={ccdf:.6g}")



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m

ot.TESTPREAMBLE()

# Create 6 points from a Halton sequence in dimension 1
expected = ot.Sample(
    [
        [1.0 / 2.0],
        [1.0 / 4.0],
        [3.0 / 4.0],
        [1.0 / 8.0],
        [5.0 / 8.0],
        [3.0 / 8.0],
    ]
)
sequence = ot.HaltonSequence(1)
print(sequence)
haltonSample = sequence.generate(6)
ott.assert_almost_equal(haltonSample, expected)

# Create 6 points from a Halton sequence in dimension 2
expected = ot.Sample(
    [
        [1.0 / 2.0, 1.0 / 3.0],
        [1.0 / 4.0, 2.0 / 3.0],
        [3.0 / 4.0, 1.0 / 9.0],
        [1.0 / 8.0, 4.0 / 9.0],
        [5.0 / 8.0, 7.0 / 9.0],
        [3.0 / 8.0, 2.0 / 9.0],
    ]
)
sequence = ot.HaltonSequence(2)
print(sequence)
haltonSample = sequence.generate(6)
ott.assert_almost_equal(haltonSample, expected)

# Create 12 points from a Halton sequence in dimension 4
expected = ot.Sample(
    [
        [0.5, 0.333333, 0.2, 0.142857],
        [0.250000, 0.666667, 0.400000, 0.285714],
        [0.750000, 0.111111, 0.600000, 0.428571],
        [0.125000, 0.444444, 0.800000, 0.571429],
        [0.625000, 0.777778, 0.040000, 0.714286],
        [0.375000, 0.222222, 0.240000, 0.857143],
        [0.875000, 0.555556, 0.440000, 0.020408],
        [0.062500, 0.888889, 0.640000, 0.163265],
        [0.562500, 0.037037, 0.840000, 0.306122],
        [0.312500, 0.370370, 0.080000, 0.448980],
        [0.812500, 0.703704, 0.280000, 0.591837],
        [0.187500, 0.148148, 0.480000, 0.734694],
    ]
)
sequence = ot.HaltonSequence(4)
print(sequence)
haltonSample = sequence.generate(12)
ott.assert_almost_equal(haltonSample, expected)

# Create 10 points from a Halton in dimension 15
sequence = ot.HaltonSequence(15)
print(sequence)
haltonSample = sequence.generate(10)

# Create another Halton' sequence of dimension 2 to estimate Pi in [0 1)^2
dimension = 2
sequence = ot.HaltonSequence(dimension)
pointInsideCircle = 0
sampleSize = (2 * 3) ** 4  # This is significant!
for i in range(sampleSize):
    haltonPoint = sequence.generate()
    if haltonPoint.norm() < 1.0:
        pointInsideCircle += 1
probabilityEstimate = (1.0 * pointInsideCircle) / sampleSize
probability = m.pi / 4.0
print("sample size=", sampleSize)
print("computed probability =", probabilityEstimate)
print("expected probability =", probability)
rtol = 10.0 / sampleSize
ott.assert_almost_equal(probability, probabilityEstimate, rtol)



"""
Function manipulation
=====================
"""

# %%
#
# In this example we are going to exhibit some of the generic function services such as:
#
# - to ask the dimension of its input and output vectors
# - to evaluate itself, its gradient and hessian
# - to set a finite difference method for the evaluation of the gradient or the hessian
# - to evaluate the number of times the function or its gradient or its hessian have been evaluated
# - to disable or enable (enabled by default) the history mechanism
# - to disable or enable the cache mechanism
# - to get all the values evaluated by the function and the associated inputs with the methods
# - to clear the history
# - to ask the description of its input and output vectors
# - to extract output components
# - to get a graphical representation

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Create a vectorial function R ^n --> R^p
# for example R^2 --> R^2
f = ot.SymbolicFunction(["x1", "x2"], ["1+2*x1+x2", "2+x1+2*x2"])

# Create a scalar function R --> R
func1 = ot.SymbolicFunction(["x"], ["x^2"])

# Create another function R^2 --> R
func2 = ot.SymbolicFunction(["x", "y"], ["x*y"])

# Create a vectorial function R ^3 --> R^2
func3 = ot.SymbolicFunction(
    ["x1", "x2", "x3"], ["1+2*x1+x2+x3^3", "2+sin(x1+2*x2)-sin(x3) * x3^4"]
)

# Create a second vectorial function R ^n --> R^p
# for example R^2 --> R^2
g = ot.SymbolicFunction(["x1", "x2"], ["x1+x2", "x1^2+2*x2^2"])


def python_eval(X):
    a, b = X
    y = a + b
    return [y]


func4 = ot.PythonFunction(2, 1, python_eval)

# %%
# Ask for the dimension of the input and output vectors
print(f.getInputDimension())
print(f.getOutputDimension())

# %%
# Enable the history mechanism
f = ot.MemoizeFunction(f)

# %%
# Evaluate the function at a particular point
x = [1.0] * f.getInputDimension()
y = f(x)
print("x=", x, "y=", y)

# %%
# Get the history
samplex = f.getInputHistory()
sampley = f.getOutputHistory()
print("evaluation history = ", samplex, sampley)

# %%
# Clear the history mechanism
f.clearHistory()

# %%
# Disable the history mechanism
f.disableHistory()

# %%
# Enable the cache mechanism
f4 = ot.MemoizeFunction(func4)
f4.enableCache()
for i in range(10):
    f4(x)

# %%
# Get the number of times cached values are reused
f4.getCacheHits()

# %%
# Evaluate the gradient of the function at a particular point
gradientMatrix = f.gradient(x)
gradientMatrix

# %%
# Evaluate the hessian of the function at a particular point
hessianMatrix = f.hessian(x)
hessianMatrix

# %%
# Change the gradient method to a non centered finite difference method
step = [1e-7] * f.getInputDimension()
gradient = ot.NonCenteredFiniteDifferenceGradient(step, f.getEvaluation())
f.setGradient(gradient)
gradient

# %%
# Change the hessian method to a centered finite difference method
step = [1e-7] * f.getInputDimension()
hessian = ot.CenteredFiniteDifferenceHessian(step, f.getEvaluation())
f.setHessian(hessian)
hessian

# %%
# Get the number of times the function has been evaluated
f.getEvaluationCallsNumber()

# %%
# Get the number of times the gradient has been evaluated
f.getGradientCallsNumber()

# %%
# Get the number of times the hessian has been evaluated
f.getHessianCallsNumber()

# %%
# Get the component i
f.getMarginal(1)

# %%
# Compose two functions : h = f o g
ot.ComposedFunction(f, g)

# %%
# Get the valid symbolic constants
ot.SymbolicFunction.GetValidConstants()

# %%
# Graph 1 : z -->  f_2(x_0,y_0,z)
# for z in [-1.5, 1.5] and y_0 = 2. and z_0 = 2.5
# Specify the input component that varies
# Care : numerotation begins at 0
inputMarg = 2
# Give its variation intervall
zMin = -1.5
zMax = 1.5
# Give the coordinates of the fixed input components
centralPt = [1.0, 2.0, 2.5]
# Specify the output marginal function
# Care : numerotation begins at 0
outputMarg = 1
# Specify the point number of the final curve
ptNb = 101
# Draw the curve!
graph = func3.draw(inputMarg, outputMarg, centralPt, zMin, zMax, ptNb)
view = otv.View(graph)

# %%
# Graph 2 : (x,z) -->  f_1(x,y_0,z)
# for x in [-1.5, 1.5], z in [-2.5, 2.5]
# and y_0 = 2.5
# Specify the input components that vary
firstInputMarg = 0
secondInputMarg = 2
# Give their variation interval
inputMin2 = [-1.5, -2.5]
inputMax2 = [1.5, 2.5]
# Give the coordinates of the fixed input components
centralPt = [0.0, 2.0, 2.5]
# Specify the output marginal function
outputMarg = 1
# Specify the point number of the final curve
ptNb = [101, 101]
graph = func3.draw(
    firstInputMarg, secondInputMarg, outputMarg, centralPt, inputMin2, inputMax2, ptNb
)
view = otv.View(graph)

# %%
# Graph 3 : simplified method for x -->  func1(x)
# for x in [-1.5, 1.5]
# Give the variation interval
xMin3 = -1.5
xMax3 = 1.5
# Specify the point number of the final curve
ptNb = 101
# Draw the curve!
graph = func1.draw(xMin3, xMax3, ptNb)
view = otv.View(graph)

# %%
# Graph 4 : (x,y) -->  func2(x,y)
# for x in [-1.5, 1.5], y in [-2.5, 2.5]
# Give their variation interval
inputMin4 = [-1.5, -2.5]
inputMax4 = [1.5, 2.5]
# Give the coordinates of the fixed input components
centralPt = [0.0, 2.0, 2.5]
# Specify the output marginal function
outputMarg = 1
# Specify the point number of the final curve
ptNb = [101, 101]
graph = func2.draw(inputMin4, inputMax4, ptNb)
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal
from openturns.usecases import ackley_function
from math import pi


ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

"""
Test the import of the AckleyModel data class.
"""
am = ackley_function.AckleyModel()

assert_almost_equal(am.dim, 2, 1e-12)
assert_almost_equal(am.a, 20.0, 1e-12)
assert_almost_equal(am.b, 0.2, 1e-12)
assert_almost_equal(am.c, 2.0 * pi, 1e-12)
assert_almost_equal(am.lowerbound, ot.Point([-15.0] * am.dim), 1e-12)
assert_almost_equal(am.upperbound, ot.Point([15.0] * am.dim), 1e-12)
assert_almost_equal(am.x0, [0.0, 0.0], 1e-12)

X = ot.Point([0.0, 0.0])
assert_almost_equal(am.model(X), [0.0], 1e-12)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

inputDimension = 3
outputDimension = 2
# Center
center = ot.Point(inputDimension)
center[0] = -1
center[1] = 0.5
center[2] = 1
# Constant term
constant = ot.Matrix(inputDimension, outputDimension)
constant[0, 0] = 1.0
constant[1, 0] = 2.0
constant[2, 0] = 5.0
constant[0, 1] = 7.0
constant[1, 1] = 9.0
constant[2, 1] = 3.0
# Linear term
linear = ot.SymmetricTensor(inputDimension, outputDimension)
linear[0, 0, 0] = 7.0
linear[0, 0, 1] = -7.0
linear[0, 1, 0] = 8.0
linear[0, 1, 1] = -8.0
linear[0, 2, 0] = 9.0
linear[0, 2, 1] = -9.0
linear[1, 0, 0] = 8.0
linear[1, 0, 1] = -8.0
linear[1, 1, 0] = 10.0
linear[1, 1, 1] = -10.0
linear[1, 2, 0] = 11.0
linear[1, 2, 1] = -11.0
linear[2, 0, 0] = 9.0
linear[2, 0, 1] = -9.0
linear[2, 1, 0] = 11.0
linear[2, 1, 1] = -11.0
linear[2, 2, 0] = 12.0
linear[2, 2, 1] = -12.0

# myFunction = linear * (X- center) + constant
myGradient = ot.LinearGradient(center, constant, linear)
myGradient.setName("linearGradient")
inPoint = ot.Point(inputDimension)
inPoint[0] = 7.0
inPoint[1] = 8.0
inPoint[2] = 9.0
outMatrix = myGradient.gradient(inPoint)
print("myGradient=", repr(myGradient))
print(myGradient.getName(), "( ", repr(inPoint), " ) = ", repr(outMatrix))



"""
Gaussian Process Regression: surrogate model with continuous and categorical variables
======================================================================================
"""

# %%
# We consider here the surrogate modeling of an analytical function characterized by
# continuous and categorical variables. Refer to :ref:`gaussian_process_regression`.
#

# %%
import openturns as ot
import numpy as np
import matplotlib.pyplot as plt

# Seed chosen in order to obtain a visually nice plot
ot.RandomGenerator.SetSeed(5)

# %%
# We first show the advantage of modeling the various levels of a mixed
# continuous / categorical function through a single surrogate model
# on a simple test-case taken from [pelamatti2020]_, defined below.


# %%
def illustrativeFunc(inp):
    x, z = inp
    y = np.cos(7 * x) + 0.5 * z
    return [y]


dim = 2
fun = ot.PythonFunction(dim, 1, illustrativeFunc)
numberOfZLevels = 2  # Number of categorical levels for z
# Input distribution
dist = ot.JointDistribution(
    [
        ot.Uniform(0, 1),
        ot.FiniteDiscreteDistribution(ot.Sample.BuildFromPoint(range(numberOfZLevels))),
    ]
)

# %%
# In this example, we compare the performances of the :class:`~openturns.LatentVariableModel`
# with a naive approach, which would consist in modeling each combination of categorical
# variables through a separate and independent Gaussian process.

# %%
# In order to deal with mixed continuous / categorical problems we can rely on the
# :class:`~openturns.ProductCovarianceModel` class. We start here by defining the product kernel,
# which combines :class:`~openturns.SquaredExponential` kernels for the continuous variables, and
# :class:`~openturns.LatentVariableModel` for the categorical ones.

# %%
latDim = 1  # Dimension of the latent space
activeCoord = 1 + latDim * (
    numberOfZLevels - 2
)  # Nb of active coordinates in the latent space
kx = ot.SquaredExponential(1)
kz = ot.LatentVariableModel(numberOfZLevels, latDim)
kLV = ot.ProductCovarianceModel([kx, kz])
kLV.setNuggetFactor(1e-6)
# Bounds for the hyperparameter optimization
lowerBoundLV = [1e-4] * dim + [-10.0] * activeCoord
upperBoundLV = [2.0] * dim + [10.0] * activeCoord
# Distribution for the hyperparameters initialization
initDistLV = ot.DistributionCollection()
for i in range(len(lowerBoundLV)):
    initDistLV.add(ot.Uniform(lowerBoundLV[i], upperBoundLV[i]))
initDistLV = ot.JointDistribution(initDistLV)

# %%
# As a reference, we consider a purely continuous kernel for independent Gaussian processes.
# One for each combination of categorical variables levels.

# %%
kIndependent = ot.SquaredExponential(1)
lowerBoundInd = [1e-4]
upperBoundInd = [20.0]
initDistInd = ot.DistributionCollection()
for i in range(len(lowerBoundInd)):
    initDistInd.add(ot.Uniform(lowerBoundInd[i], upperBoundInd[i]))
initDistInd = ot.JointDistribution(initDistInd)
initSampleInd = initDistInd.getSample(10)
optAlgInd = ot.MultiStart(ot.Cobyla(), initSampleInd)

# %%
# Generate the training data set
x = dist.getSample(10)
y = fun(x)

# And the plotting data set
xPlt = dist.getSample(200)
xPlt = xPlt.sort()
yPlt = fun(xPlt)

# %%
# Initialize  and parameterize the optimization algorithm
initSampleLV = initDistLV.getSample(30)
optAlgLV = ot.MultiStart(ot.Cobyla(), initSampleLV)

# %%
# Create and train the Gaussian process models
basis = ot.ConstantBasisFactory(2).build()
fitterLV = ot.GaussianProcessFitter(x, y, kLV, basis)
fitterLV.setOptimizationAlgorithm(optAlgLV)
fitterLV.run()
regressionLV = ot.GaussianProcessRegression(fitterLV.getResult())
regressionLV.run()
resLV = regressionLV.getResult()

resIndependentList = []
for z in range(2):
    # Select the training samples corresponding to the correct combination
    # of categorical levels
    ind = np.where(np.all(np.array(x[:, 1]) == z, axis=1))[0]
    xLoc = x[ind][:, 0]
    yLoc = y[ind]

    # Create and train the Gaussian process models
    basis = ot.ConstantBasisFactory(1).build()
    fitter_independent = ot.GaussianProcessFitter(xLoc, yLoc, kIndependent, basis)
    fitter_independent.setOptimizationAlgorithm(optAlgInd)
    fitter_independent.run()
    regression_independent = ot.GaussianProcessRegression(
        fitter_independent.getResult()
    )
    regression_independent.run()
    resIndependentList.append(regression_independent.getResult())

# %%
# Plot the prediction of the mixed continuous / categorical GP,
# as well as the one of the two separate continuous GPs
fig, (ax1, ax2) = plt.subplots(2, 1, sharex=True, figsize=(15, 10))
for z in range(numberOfZLevels):
    # Select the training samples corresponding to the correct combination
    # of categorical levels
    ind = np.where(np.all(np.array(x[:, 1]) == z, axis=1))[0]
    xLoc = x[ind][:, 0]
    yLoc = y[ind]

    # Compute the models predictive performances on a validation data set.
    # The predictions are computed independently for each level of z,
    # i.e., by only considering the values of z corresponding to the
    # target level.
    ind = np.where(np.all(np.array(xPlt[:, 1]) == z, axis=1))[0]
    xPltInd = xPlt[ind]
    yPltInd = yPlt[ind]

    predMeanLV = resLV.getMetaModel()(xPltInd)
    predMeanInd = resIndependentList[z].getMetaModel()(xPltInd[:, 0])
    cond_covLV = ot.GaussianProcessConditionalCovariance(resLV)
    cond_independent = ot.GaussianProcessConditionalCovariance(resIndependentList[z])
    predSTDLV = np.sqrt(cond_covLV.getConditionalMarginalVariance(xPltInd))
    predSTDInd = np.sqrt(cond_independent.getConditionalMarginalVariance(xPltInd[:, 0]))

    (trainingData,) = ax1.plot(xLoc[:, 0], yLoc, "r*")
    (trueFunction,) = ax1.plot(xPltInd[:, 0], yPltInd, "k--")
    (prediction,) = ax1.plot(xPltInd[:, 0], predMeanLV, "b-")
    stdPred = ax1.fill_between(
        xPltInd[:, 0].asPoint(),
        (predMeanLV - predSTDLV).asPoint(),
        (predMeanLV + predSTDLV).asPoint(),
        alpha=0.5,
        color="blue",
    )
    ax2.plot(xLoc[:, 0], yLoc, "r*")
    ax2.plot(xPltInd[:, 0], yPltInd, "k--")
    ax2.plot(xPltInd[:, 0], predMeanInd, "b-")
    ax2.fill_between(
        xPltInd[:, 0].asPoint(),
        (predMeanInd - predSTDInd).asPoint(),
        (predMeanInd + predSTDInd).asPoint(),
        alpha=0.5,
        color="blue",
    )
ax1.legend(
    [trainingData, trueFunction, prediction, stdPred],
    ["Training data", "True function", "Prediction", "Prediction standard deviation"],
)
ax1.set_title("Mixed continuous-categorical modeling")
ax2.set_title("Separate modeling")
ax2.set_xlabel("x", fontsize=15)
ax1.set_ylabel("y", fontsize=15)
ax2.set_ylabel("y", fontsize=15)

# %%
# It can be seen that the joint modeling of categorical and continuous variables
# improves the overall prediction accuracy, as the Gaussian process model is
# able to exploit the information provided by the entire training data set.

# %%
# We now consider a more complex function which is a modified version of the Goldstein function,
# taken from [pelamatti2020]_. This function depends on 2 continuous variables and 2 categorical ones.
# Each categorical variable is characterized by 3 levels.


# %%
def h(x1, x2, x3, x4):
    y = (
        53.3108
        + 0.184901 * x1
        - 5.02914 * x1**3 * 1e-6
        + 7.72522 * x1**4 * 1e-8
        - 0.0870775 * x2
        - 0.106959 * x3
        + 7.98772 * x3**3 * 1e-6
        + 0.00242482 * x4
        + 1.32851 * x4**3 * 1e-6 * 0.00146393 * x1 * x2
        - 0.00301588 * x1 * x3
        - 0.00272291 * x1 * x4
        + 0.0017004 * x2 * x3
        + 0.0038428 * x2 * x4
        - 0.000198969 * x3 * x4
        + 1.86025 * x1 * x2 * x3 * 1e-5
        - 1.88719 * x1 * x2 * x4 * 1e-6
        + 2.50923 * x1 * x3 * x4 * 1e-5
        - 5.62199 * x2 * x3 * x4 * 1e-5
    )
    return y


def Goldstein(inp):
    x1, x2, z1, z2 = inp
    x1 = 100 * x1
    x2 = 100 * x2

    if z1 == 0:
        x3 = 80
    elif z1 == 1:
        x3 = 20
    elif z1 == 2:
        x3 = 50
    else:
        print("error, no matching category z1")

    if z2 == 0:
        x4 = 20
    elif z2 == 1:
        x4 = 80
    elif z2 == 2:
        x4 = 50
    else:
        print("error, no matching category z2")

    return [h(x1, x2, x3, x4)]


dim = 4
fun = ot.PythonFunction(dim, 1, Goldstein)
numberOfZLevels1 = 3  # Number of categorical levels for z1
numberOfZLevels2 = 3  # Number of categorical levels for z2
# Input distribution
dist = ot.JointDistribution(
    [
        ot.Uniform(0, 1),
        ot.Uniform(0, 1),
        ot.FiniteDiscreteDistribution(
            ot.Sample.BuildFromPoint(range(numberOfZLevels1))
        ),
        ot.FiniteDiscreteDistribution(
            ot.Sample.BuildFromPoint(range(numberOfZLevels2))
        ),
    ]
)

# %%
# As in the previous example, we start here by defining the product kernel,
# which combines :class:`~openturns.SquaredExponential` kernels for the continuous variables, and
# :class:`~openturns.LatentVariableModel` for the categorical ones.

# %%
latDim = 2  # Dimension of the latent space
activeCoord = (
    2 + latDim * (numberOfZLevels1 - 2) + latDim * (numberOfZLevels2 - 2)
)  # Nb ative coordinates in the latent space
kx1 = ot.SquaredExponential(1)
kx2 = ot.SquaredExponential(1)
kz1 = ot.LatentVariableModel(numberOfZLevels1, latDim)
kz2 = ot.LatentVariableModel(numberOfZLevels2, latDim)
kLV = ot.ProductCovarianceModel([kx1, kx2, kz1, kz2])
kLV.setNuggetFactor(1e-6)
# Bounds for the hyperparameter optimization
lowerBoundLV = [1e-4] * dim + [-10] * activeCoord
upperBoundLV = [3.0] * dim + [10.0] * activeCoord
# Distribution for the hyperparameters initialization
initDistLV = ot.DistributionCollection()
for i in range(len(lowerBoundLV)):
    initDistLV.add(ot.Uniform(lowerBoundLV[i], upperBoundLV[i]))
initDistLV = ot.JointDistribution(initDistLV)

# %%
# Alternatively, we consider a purely continuous kernel for independent Gaussian processes.
# one for each combination of categorical variables levels.

# %%
kIndependent = ot.SquaredExponential(2)
lowerBoundInd = [1e-4, 1e-4]
upperBoundInd = [3.0, 3.0]
initDistInd = ot.DistributionCollection()
for i in range(len(lowerBoundInd)):
    initDistInd.add(ot.Uniform(lowerBoundInd[i], upperBoundInd[i]))
initDistInd = ot.JointDistribution(initDistInd)
initSampleInd = initDistInd.getSample(10)
optAlgInd = ot.MultiStart(ot.Cobyla(), initSampleInd)

# %%
# In order to assess their respective robustness with regards to the training data set,
# we repeat the experiments 3 times with different training of size 72,
# and compute each time the normalized prediction Root Mean Squared Error (RMSE) on a
# test data set of size 1000.
rmseLVList = []
rmseIndList = []
for rep in range(3):
    # Generate the normalized training data set
    x = dist.getSample(72)
    y = fun(x)
    yMax = y.getMax()
    yMin = y.getMin()
    y = (y - yMin) / (yMin - yMax)

    # Initialize and parameterize the optimization algorithm
    initSampleLV = initDistLV.getSample(10)
    optAlgLV = ot.MultiStart(ot.Cobyla(), initSampleLV)

    # Create and train the Gaussian process models
    basis = ot.ConstantBasisFactory(dim).build()
    fitterLV = ot.GaussianProcessFitter(x, y, kLV, basis)
    fitterLV.setOptimizationAlgorithm(optAlgLV)
    fitterLV.run()
    regressionLV = ot.GaussianProcessRegression(fitterLV.getResult())
    regressionLV.run()
    resLV = regressionLV.getResult()

    # Compute the models predictive performances on a validation data set
    xVal = dist.getSample(1000)
    yVal = fun(xVal)
    yVal = (yVal - yMin) / (yMin - yMax)

    valLV = ot.MetaModelValidation(yVal, resLV.getMetaModel()(xVal))
    rmseLV = valLV.getResidualSample().computeStandardDeviation()[0]
    rmseLVList.append(rmseLV)

    error = ot.Sample(0, 1)
    for z1 in range(numberOfZLevels1):
        for z2 in range(numberOfZLevels2):
            # Select the training samples corresponding to the correct combination
            # of categorical levels
            ind = np.where(np.all(np.array(x[:, 2:]) == [z1, z2], axis=1))[0]
            xLoc = x[ind][:, :2]
            yLoc = y[ind]

            # Create and train the Gaussian process models
            basis = ot.ConstantBasisFactory(2).build()
            fitter_independent = ot.GaussianProcessFitter(
                xLoc, yLoc, kIndependent, basis
            )
            fitter_independent.setOptimizationAlgorithm(optAlgInd)
            fitter_independent.run()
            regression_independent = ot.GaussianProcessRegression(
                fitter_independent.getResult()
            )
            regression_independent.run()
            resInd = regression_independent.getResult()

            # Compute the models predictive performances on a validation data set
            ind = np.where(np.all(np.array(xVal[:, 2:]) == [z1, z2], axis=1))[0]
            xValInd = xVal[ind][:, :2]
            yValInd = yVal[ind]
            valInd = ot.MetaModelValidation(yValInd, resInd.getMetaModel()(xValInd))
            error.add(valInd.getResidualSample())
    rmseInd = error.computeStandardDeviation()[0]
    rmseIndList.append(rmseInd)

plt.figure()
plt.boxplot([rmseLVList, rmseIndList])
plt.xticks([1, 2], ["Mixed continuous-categorical GP", "Independent GPs"])
plt.ylabel("RMSE")

# %%
# The obtained results show, for this test-case, a better modeling performance
# when modeling the function as a mixed categorical/continuous function, rather
# than relying on multiple purely continuous Gaussian processes.
plt.show(block=True)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Set Numerical precision to 4
ot.PlatformInfo.SetNumericalPrecision(4)
sampleSize = 40
inputDimension = 1

# Create the function to estimate
model = ot.SymbolicFunction(["x0"], ["x0"])

X = ot.Sample(sampleSize, inputDimension)
for i in range(sampleSize):
    X[i, 0] = 3.0 + (8.0 * i) / sampleSize
Y = model(X)

# Add a small noise to data
Y += (
    ot.GaussianProcess(ot.AbsoluteExponential([0.1], [0.2]), ot.Mesh(X))
    .getRealization()
    .getValues()
)

basis = ot.LinearBasisFactory(inputDimension).build()
# Case of a misspecified covariance model
covarianceModel = ot.DiracCovarianceModel(inputDimension)
print("===================================================\n")
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis)
algo.run()

result = algo.getResult()
print("\ncovariance (dirac, optimized)=", result.getCovarianceModel())
print("trend (dirac, optimized)=", result.getTrendCoefficients())
print("===================================================\n")

# Now without estimating covariance parameters
basis = ot.LinearBasisFactory(inputDimension).build()
covarianceModel = ot.DiracCovarianceModel(inputDimension)
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis, True)
algo.setOptimizeParameters(False)
algo.run()
result = algo.getResult()
print("\ncovariance (dirac, not optimized)=", result.getCovarianceModel())
print("trend (dirac, not optimized)=", result.getTrendCoefficients())
print("===================================================\n")
# Now without estimating covariance parameters
basis = ot.LinearBasisFactory(inputDimension).build()
covarianceModel = ot.DiracCovarianceModel(inputDimension)
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis, True)
algo.setOptimizeParameters(False)
algo.run()
result = algo.getResult()
print("\ncovariance (dirac, not optimized)=", result.getCovarianceModel())
print("trend (dirac, not optimized)=", result.getTrendCoefficients())
print("===================================================\n")

# Case of a well specified covariance model
# Test the optimization when the amplitude is deduced analytically from
# the scale
covarianceModel = ot.AbsoluteExponential(inputDimension)
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis)
algo.run()
result = algo.getResult()
print("\ncovariance (reduced, unbiased)=", result.getCovarianceModel())
print("trend (reduced, unbiased)=", result.getTrendCoefficients())
print("===================================================\n")
ot.ResourceMap.SetAsBool("GeneralLinearModelAlgorithm-UnbiasedVariance", False)
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis)
algo.run()
result = algo.getResult()
print("\ncovariance (reduced, biased)=", result.getCovarianceModel())
print("trend (reduced, biased)=", result.getTrendCoefficients())
print("===================================================\n")
ot.ResourceMap.SetAsBool(
    "GeneralLinearModelAlgorithm-UseAnalyticalAmplitudeEstimate", False
)
algo = ot.GeneralLinearModelAlgorithm(X, Y, covarianceModel, basis)
# Define interval
bounds = ot.Interval([1e-2] * 2, [100] * 2)
algo.setOptimizationBounds(bounds)
algo.run()
result = algo.getResult()
print("\ncovariance (full optim)=", result.getCovarianceModel())
print("trend (full optim)=", result.getTrendCoefficients())
print("===================================================\n")



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# The 1D interface
interval1D = ot.Interval(-3, 5)
print("interval1D=", interval1D)
# The default interface
size = 2
defaultInterval = ot.Interval(2)
print("defaultInterval=", repr(defaultInterval))
# Construction from 2 Point
p1 = ot.Point(2, -1.0)
p2 = ot.Point(2, 2.0)
print("interval from 2 points=", repr(ot.Interval(p1, p2)))
print("pretty print=")
print(ot.Interval(p1, p2))
# Construction from 2 points and 2 flags
flag1 = ot.BoolCollection(2, False)
flag2 = [True, True]  # try automatic conversion to BoolCollection
interval = ot.Interval(p1, p2, flag1, flag2)
print("interval from 2 points and 2 flags=", repr(interval))
print("pretty print=")
print(interval)
# Accessors
print("lower bound=", repr(interval.getLowerBound()))
print("upper bound=", repr(interval.getUpperBound()))
print("lower bound flags=", repr(interval.getFiniteLowerBound()))
print("upper bound flags=", repr(interval.getFiniteUpperBound()))
# Check if a given interval is empty
print("interval [p1, p2] empty? ", ot.Interval(p1, p2).isEmpty())
print("interval [p2, p1] empty? ", ot.Interval(p2, p1).isEmpty())
# Intersection
interval1 = ot.Interval(p1, p2)
p3 = ot.Point(2)
p3[0] = 0.5
p3[1] = -1.5
p4 = ot.Point(2)
p4[0] = 1.5
p4[1] = 2.5
interval2 = ot.Interval(p3, p4)
print(
    "intersection of ",
    repr(interval1),
    " and ",
    repr(interval2),
    " equals ",
    repr(interval1.intersect(interval2)),
)
# Inside
p5 = (p1 + p2) * 0.5
print("is point", p5, "inside", repr(interval1), "? ", p5 in interval1)
p6 = p2 * 2.0
print("is point", p6, "inside", repr(interval1), "? ", p6 in interval1)
print("marginal(1)=", interval2.getMarginal(1))
print("marginal(1,0)=", interval2.getMarginal([1, 0]))



import openturns as ot
import openturns.viewer as otv

lowerBound = [0.0, 0.0]
upperBound = [1.0, 1.0]
interv = ot.Interval(lowerBound, upperBound)
mesher = ot.IntervalMesher([6] * 2)
mesh = mesher.build(interv, True)

f = ot.SymbolicFunction(["x", "y"], ["sin(pi_*x)*sin(pi_*y)"])
values = f(mesh.getVertices())

field = ot.Field(mesh, values)
func_P1 = ot.P1LagrangeEvaluation(field)

graph = mesh.draw()
graph.setColors(["lightblue"])
draw_f = f.draw(lowerBound, upperBound, [1024] * 2).getDrawable(0)
draw_f.setLineStyle("dashed")
lev = draw_f.getLevels()
draw_fP1 = (
    func_P1.draw(lowerBound, upperBound, [1024] * 2).getDrawable(0).getImplementation()
)
draw_fP1.setLevels(lev)
draw_fP1.setColorBarPosition("")
graph.add(draw_f)
graph.add(draw_fP1)
graph.setTitle(
    r"P1 Lagrange interpolation (solid) of $(x,y) \mapsto \sin(\pi x) \sin(\pi y)$ (dashed) from a mesh (blue solid)"
)
graph.setXTitle("$x$")
graph.setYTitle("$y$")

view = otv.View(graph, figure_kw={"figsize": (10, 4)}, add_legend=False)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

dim = 2
interval = ot.Interval([0.0] * dim, [10.0] * dim)
mesh = ot.IntervalMesher([30] * dim).build(interval)
f = ot.SymbolicFunction(["x", "y"], ["x + 0.5*sin(y)", "y-0.1*x*sin(x)"])
mesh.setVertices(f(mesh.getVertices()))

simplices = mesh.getSimplices()
nrSimplices = len(simplices)
naive = ot.NaiveEnclosingSimplex(mesh.getVertices(), simplices)
print("naive=", naive)

ot.RandomGenerator.SetSeed(0)
test = ot.JointDistribution([ot.Uniform(-1.0, 11.0)] * dim).getSample(100)

for i, vertex in enumerate(test):
    index = naive.query(vertex)
    if index >= nrSimplices:
        print(i, "is outside")
    else:
        found, coordinates = mesh.checkPointInSimplexWithCoordinates(vertex, index)
        if not found:
            print(
                "Wrong simplex found for",
                vertex,
                "(index=",
                index,
                simplices[index],
                "barycentric coordinates=",
                coordinates,
            )

indices = naive.query(test)
for i, index in enumerate(indices):
    if index >= nrSimplices:
        print(i, "is outside")
    else:
        found, coordinates = mesh.checkPointInSimplexWithCoordinates(test[i], index)
        if not found:
            print(
                "Wrong simplex found for",
                test[i],
                "(index=",
                index,
                simplices[index],
                "barycentric coordinates=",
                coordinates,
            )



#! /usr/bin/env python

import openturns as ot
from openturns.usecases import ishigami_function

ot.TESTPREAMBLE()

input_dimension = 3
output_dimension = 1

formula = [
    "sin(pi_*X1)+7*sin(pi_*X2)*sin(pi_*X2)+0.1*((pi_*X3)*(pi_*X3)*(pi_*X3)*(pi_*X3))*sin(pi_*X1)"
]

model = ot.SymbolicFunction(["X1", "X2", "X3"], formula)

distribution = ot.JointDistribution([ot.Uniform(-1.0, 1.0)] * input_dimension)

# Size of simulation
size = 10000

# Test with the various implementation methods
methods = ["Saltelli", "Jansen", "MauntzKucherenko", "Martinez"]

# Test the different sampling options
samplings = ["MonteCarlo", "LHS", "QMC"]

# Generate input/output designs
computeSO = True
# Case 1 : Estimation of sensitivity using estimator and no bootstrap
for method in methods:
    for sampling in samplings:
        ot.ResourceMap.SetAsString("SobolIndicesExperiment-SamplingMethod", sampling)
        sensitivity_algorithm = eval(
            "ot."
            + method
            + "SensitivityAlgorithm(distribution, size, model, computeSO)"
        )
        print("Method of evaluation=", method)
        print("Method of sampling=", sampling)
        # Get first order indices
        fo = sensitivity_algorithm.getFirstOrderIndices()
        print("First order indices = ", fo)
        # Get total order indices
        to = sensitivity_algorithm.getTotalOrderIndices()
        print("Total order indices = ", to)

        # Get the confidence interval thanks to Bootstrap
        nr_bootstrap = 100
        confidence_level = 0.95
        sensitivity_algorithm.setBootstrapSize(nr_bootstrap)
        sensitivity_algorithm.setConfidenceLevel(confidence_level)
        sensitivity_algorithm.setUseAsymptoticDistribution(False)
        interval_fo = sensitivity_algorithm.getFirstOrderIndicesInterval()
        interval_to = sensitivity_algorithm.getTotalOrderIndicesInterval()
        print("bootstrap intervals")
        print("First order indices interval = ", interval_fo)
        print("Total order indices interval = ", interval_to)

        # Asymptotic confidence interval
        sensitivity_algorithm.setUseAsymptoticDistribution(True)
        interval_fo_asymptotic = sensitivity_algorithm.getFirstOrderIndicesInterval()
        interval_to_asymptotic = sensitivity_algorithm.getTotalOrderIndicesInterval()
        print("asymptotic intervals:")
        print(
            "First order indices distribution = ",
            sensitivity_algorithm.getFirstOrderIndicesDistribution(),
        )
        print(
            "Total order indices distribution = ",
            sensitivity_algorithm.getTotalOrderIndicesDistribution(),
        )
        print("First order indices interval = ", interval_fo_asymptotic)
        print("Total order indices interval = ", interval_to_asymptotic)

# with experiment
sequence = ot.SobolSequence(input_dimension)
experiment = ot.LowDiscrepancyExperiment(
    sequence, ot.JointDistribution([ot.Uniform(0.0, 1.0)] * input_dimension), size
)
sensitivity_algorithm = ot.SaltelliSensitivityAlgorithm(experiment, model)
print(sensitivity_algorithm.getFirstOrderIndices())

# multi variate model
model_aggregated = ot.SymbolicFunction(
    ["X1", "X2", "X3"],
    ["2*X1 + X2 - 3*X3 + 0.3*X1*X2", "-5*X1 + 4*X2 - 0.8*X2*X3 + 2*X3"],
)
distribution_aggregated = ot.JointDistribution([ot.Uniform()] * 3)
inputDesign = ot.SobolIndicesExperiment(distribution_aggregated, size).generate()
outputDesign = model_aggregated(inputDesign)
# Case 1 : Estimation of sensitivity using estimator and no bootstrap
for method in methods:
    sensitivity_algorithm = eval(
        "ot." + method + "SensitivityAlgorithm(inputDesign, outputDesign, size)"
    )
    print("Method of evaluation=", method)

    # Get first order indices
    fo = sensitivity_algorithm.getAggregatedFirstOrderIndices()
    print("Aggregated first order indices = ", fo)
    # Get total order indices
    to = sensitivity_algorithm.getAggregatedTotalOrderIndices()
    print("Aggregated total order indices = ", to)

    # Get the confidence interval thanks to Bootstrap
    nr_bootstrap = 100
    confidence_level = 0.95
    # sensitivity_algorithm = ot.MartinezSensitivityAlgorithm(
    # inputDesign, outputDesign, size)
    sensitivity_algorithm.setBootstrapSize(nr_bootstrap)
    sensitivity_algorithm.setConfidenceLevel(confidence_level)
    sensitivity_algorithm.setUseAsymptoticDistribution(False)
    interval_fo = sensitivity_algorithm.getFirstOrderIndicesInterval()
    interval_to = sensitivity_algorithm.getTotalOrderIndicesInterval()
    print("bootstrap intervals")
    print("Aggregated first order indices interval = ", interval_fo)
    print("Aggregated total order indices interval = ", interval_to)

    # Asymptotic confidence interval
    sensitivity_algorithm.setUseAsymptoticDistribution(True)
    interval_fo_asymptotic = sensitivity_algorithm.getFirstOrderIndicesInterval()
    interval_to_asymptotic = sensitivity_algorithm.getTotalOrderIndicesInterval()
    print("asymptotic intervals:")
    print("Aggregated first order indices interval = ", interval_fo_asymptotic)
    print("Aggregated total order indices interval = ", interval_to_asymptotic)


# special case in dim=2
ot.ResourceMap.SetAsString("SobolIndicesExperiment-SamplingMethod", "MonteCarlo")
ot.RandomGenerator.SetSeed(0)
distribution = ot.JointDistribution([ot.Uniform()] * 2)
size = 1000
model = ot.SymbolicFunction(["X1", "X2"], ["2*X1 + X2"])
sensitivity_algorithm = ot.SaltelliSensitivityAlgorithm(distribution, size, model, True)
print(sensitivity_algorithm.getSecondOrderIndices())
ot.RandomGenerator.SetSeed(0)
experiment = ot.MonteCarloExperiment(distribution, size)
sensitivity_algorithm = ot.SaltelliSensitivityAlgorithm(experiment, model, True)
print(sensitivity_algorithm.getSecondOrderIndices())
ot.RandomGenerator.SetSeed(0)
x = ot.SobolIndicesExperiment(distribution, size, True).generate()
y = model(x)
sensitivity_algorithm = ot.SaltelliSensitivityAlgorithm(x, y, size)
print(sensitivity_algorithm.getSecondOrderIndices())


# null contribution case: X3 not in output formula
model = ot.SymbolicFunction(["X1", "X2", "X3"], ["10+3*X1+X2"])
distribution = ot.JointDistribution([ot.Uniform(-1.0, 1.0)] * input_dimension)
size = 10000
for method in methods:
    sensitivity_algorithm = eval(
        "ot." + method + "SensitivityAlgorithm(distribution, size, model, False)"
    )
    sensitivity_algorithm.setUseAsymptoticDistribution(True)
    fo = sensitivity_algorithm.getFirstOrderIndices()
    print("First order indices = ", fo)
    to = sensitivity_algorithm.getTotalOrderIndices()
    # print("Total order indices = ", to)
    interval_fo = sensitivity_algorithm.getFirstOrderIndicesInterval()
    interval_to = sensitivity_algorithm.getTotalOrderIndicesInterval()
    print("Aggregated first order indices interval = ", interval_fo)
    # print("Aggregated total order indices interval = ", interval_to)

# setDesign must reset results across runs
ot.Log.Show(ot.Log.NONE)
im = ishigami_function.IshigamiModel()
exact_first_order = ot.Point([im.S1, im.S2, im.S3])
exact_total_order = ot.Point([im.ST1, im.ST2, im.ST3])
sobolIndicesAlgorithmB = ot.SaltelliSensitivityAlgorithm()
for sample_size in [100, 1000, 10000]:
    print("Size:", sample_size)

    # Method A : classical
    X = ot.SobolIndicesExperiment(im.distribution, sample_size).generate()
    Y = im.model(X)
    sobolIndicesAlgorithmA = ot.SaltelliSensitivityAlgorithm(X, Y, sample_size)
    computed_first_orderA = sobolIndicesAlgorithmA.getFirstOrderIndices()
    computed_total_orderA = sobolIndicesAlgorithmA.getTotalOrderIndices()
    first_error = computed_first_orderA - exact_first_order
    total_error = computed_total_orderA - exact_total_order
    print("Method A :", first_error, total_error)

    # Method B : setDesign
    sobolIndicesAlgorithmB.setDesign(X, Y, sample_size)
    computed_first_orderB = sobolIndicesAlgorithmB.getFirstOrderIndices()
    computed_total_orderB = sobolIndicesAlgorithmB.getTotalOrderIndices()

    first_error = computed_first_orderB - exact_first_order
    total_error = computed_total_orderB - exact_total_order
    print("Method B :", first_error, total_error)
    assert computed_first_orderB == computed_first_orderA, "wrong first"
    assert computed_total_orderB == computed_total_orderA, "wrong total"



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


inVar = ["t"]
functions = []
functions.append(ot.SymbolicFunction(inVar, ["1"]))
functions.append(ot.SymbolicFunction(inVar, ["cos(2 * t)"]))
functions.append(ot.SymbolicFunction(inVar, ["sin(2 * t)"]))

# We build the weights
coefficients = ot.Sample(0, 2)
p = ot.Point(2)
p[0] = 1.5
p[1] = 2.5
coefficients.add(p)
p[0] = -0.5
p[1] = 0.5
coefficients.add(p)
p[0] = 1.0
p[1] = 1.0
coefficients.add(p)

# Third, build the function
myFunction = ot.DualLinearCombinationFunction(functions, coefficients)

# Fourth : we build a time series for estimation
# it issued from a white noise
dimension = 2

# Fix the realization as a Normal
noiseDistribution = ot.Normal(dimension)

# TimeGrid parameters
N = 1000
timeStart = 0.0
timeStep = 0.1
timeGrid = ot.RegularGrid(timeStart, timeStep, N)

# White noise
myWhiteNoise = ot.WhiteNoise(noiseDistribution, timeGrid)

realization = ot.TimeSeries(myWhiteNoise.getRealization())
print("White noise realization = ", realization)

# We make a trend transform to the time series
# We get a time series which contains values of time
myTransformFunction = ot.TrendTransform(myFunction, timeGrid)
myTimeSeries = ot.TimeSeries(timeGrid, myTransformFunction(realization.getValues()))
print("myTimeSeries = ", myTimeSeries)

# We wants to get the coefficients using a factory
# Build a factory using default constructor
myDefaultFactory = ot.TrendFactory()
print("myDefaultFactory = ", myDefaultFactory)

myEstimateTrend = myDefaultFactory.build(myTimeSeries, ot.Basis(functions))
print("myEstimateTrend = ", myEstimateTrend)

# We fix a new fitting algorithm
myDefaultFactory.setFittingAlgorithm(ot.KFold())
print("myDefaultFactory = ", myDefaultFactory)
myNewEstimateTrend = myDefaultFactory.build(myTimeSeries, ot.Basis(functions))
print("myNewEstimateTrend = ", myNewEstimateTrend)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math

ot.RandomGenerator.SetSeed(0)

ot.TESTPREAMBLE()

# %%
# Test RandomVectorMetropolisHastings on Beta-Binomial conjugate model

# Define Beta-binomial model
a, b, lower, upper = 1.0, 1.0, 0.0, 1.0
n, p = 10, 0.5
prior = ot.RandomVector(ot.Beta(a, b, lower, upper))
model = ot.Binomial(n, p)

# %%
# Simulate data and compute analytical posterior
x = model.getSample(1)
posterior = ot.Beta(a + x[0, 0], b + n - x[0, 0], lower, upper)

# %%
# Define RVMH sampler
# Here, the random vector is defined with repsect to the prior law
# this effectively implements the IMH algorithm using the prior
# as an instrumental distribution
initialState = [p]
rvmh_sampler = ot.RandomVectorMetropolisHastings(prior, initialState, [0])
slf = ot.SymbolicFunction(["x"], [str(n), "x"])
rvmh_sampler.setLikelihood(model, x, slf)

# %%
# Generate posterior distribution sample
sampleSize = 10000
xSample = rvmh_sampler.getSample(sampleSize)

# %%
# Compare empirical to theoretical moments

ott.assert_almost_equal(
    xSample.computeMean(), posterior.getMean(), 0.0, 10.0 / math.sqrt(sampleSize)
)
ott.assert_almost_equal(
    xSample.computeStandardDeviation(),
    posterior.getStandardDeviation(),
    0.0,
    10.0 / math.sqrt(sampleSize),
)

# %%
randomVector = ot.RandomVector(ot.Normal())
initialState = [0.0]
sampler = ot.RandomVectorMetropolisHastings(randomVector, initialState)
x = sampler.getSample(10000)
mean = x.computeMean()
stddev = x.computeStandardDeviation()
ott.assert_almost_equal(mean, [0.0], 0.0, 0.03)
ott.assert_almost_equal(stddev, [1.0], 0.03, 0.0)

# with link function
slf = ot.SymbolicFunction(["x"], ["0.5", "0.1"])
sampler = ot.RandomVectorMetropolisHastings(randomVector, initialState, [0], slf)
x = sampler.getSample(10000)
mean = x.computeMean()
stddev = x.computeStandardDeviation()
ott.assert_almost_equal(mean, [0.5], 0.0, 0.003)
ott.assert_almost_equal(stddev, [0.1], 0.03, 0.0)



# Tool to compare testsuite costs
#
# 1. run tests in reference configuration:
# $ make install && make tests && rm -rf Testing && ctest && cp Testing/Temporary/CTestCostData.txt CTestCostData0.txt
#
# 2. run tests with new code:
# $ make install && make tests && rm -rf Testing && ctest && cp Testing/Temporary/CTestCostData.txt CTestCostData1.txt
#
# 3. compare tests costs:
# $ python3 ../utils/costDiff.py CTestCostData0.txt CTestCostData1.txt


import argparse

def parse_cost(path):
    tdata = dict()
    with open(path) as cost1:
        for line in cost1.readlines():
            try:
                tag, reps, cost = line.split(" ")
                if reps == "0":
                    raise ValueError()
                tdata[tag] = float(cost)
            except ValueError:
                pass
    return tdata

def compare_cost(path1, path2):
    data1 = parse_cost(path1)
    data2 = parse_cost(path2)
    diffData = {}
    relDiffData = {}
    n_slower = 0
    n_faster = 0
    a_tol = 8e-1
    r_tol = 1e-1

    for key in data1.keys():
        if data1[key] < a_tol:
            continue
        if key in data2:
            diff = data2[key] - data1[key]
            diffData[key] = diff
            relDiff = diff / data1[key]
            if diff > a_tol and relDiff > r_tol:
                n_slower +=1
            if diff < -a_tol and -relDiff > r_tol:
                n_faster +=1
            relDiffData[key] = relDiff

    for key in diffData.keys():
        print(f"{key} diff {diffData[key]:.3f} slowdown {relDiffData[key]:.3f}")

    n = 10
    print(f"= top {n} SLOWEST tests ============")
    std = sorted(data1.items(), key=lambda item: item[1])
    slowest = dict(reversed(std[-n:]))
    for key in slowest.keys():
        print(f"{key} {data1[key]:.3f}")

    std = sorted(relDiffData.items(), key=lambda item: item[1])
    slower = dict(reversed(std[-n:]))
    faster = dict(std[:n])
    print(f"= top {n} FASTER tests ============")
    for key in faster.keys():
        print(f"{key} diff {diffData[key]:.3f} speedup {-100.0*relDiffData[key]:.3f}% (before={data1[key]:.3f} after={data2[key]:.3f})")

    print(f"= top {n} SLOWER tests ============")
    for key in slower.keys():
        print(f"{key} diff {diffData[key]:.3f} slowdown {100.0*relDiffData[key]:.3f}% (before={data1[key]:.3f} after={data2[key]:.3f})")

    print(f"==============")
    print(f"n_faster={n_faster} n_slower={n_slower}")
    print(f"==============")

def main():
    """
    entry point.
    """
    parser = argparse.ArgumentParser(description="Compare test data")
    parser.add_argument("path1", type=str, help="Path to the cost data (eg CTestCostData.txt)")
    parser.add_argument("path2", type=str, help="Path to the cost data (eg CTestCostData.txt)")
    args = parser.parse_args()
    compare_cost(**vars(args))

if __name__ == "__main__":
    main()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


#  Constructor with size
dimension = 2
squareMatrix1 = ot.SquareMatrix(dimension)

squareMatrix1[0, 0] = 1.0
squareMatrix1[1, 0] = 2.0
squareMatrix1[0, 1] = 3.0
squareMatrix1[1, 1] = 4.0

# Second matrix to add to the ARMACoefficients
s = 3.0
squareMatrix2 = squareMatrix1 * s

# Second matrix to add to the ARMACoefficients
t = 1.5
squareMatrix3 = squareMatrix1 / t

# size : Number of matrix
size = 3

# ARMACoefficients with default constructor
coefficients0 = ot.ARMACoefficients()
print("Using default constructor ")
print("coefficients0 = ", coefficients0)

# ARMACoefficients with size / dimension constructor
coefficients1 = ot.ARMACoefficients(size, dimension)
coefficients1[0] = squareMatrix1
coefficients1[1] = squareMatrix2
coefficients1[2] = squareMatrix3

print("Using constructor based on size / dimension ")
print("coefficients1 = ", repr(coefficients1))

# constructors with collection of matrix
myCollection = ot.SquareMatrixCollection()
myCollection.add(squareMatrix1)
myCollection.add(squareMatrix2)
myCollection.add(squareMatrix3)

print("SquareMatrix collection = ", repr(myCollection))
coefficients2 = ot.ARMACoefficients(myCollection)

# print  of the new collection
print("Using constructor based on a collection ")
print("coefficients2 = ", repr(coefficients2))

# 1D case
point = ot.Point(dimension * dimension, 0.0)
point[0] = 1.0
point[1] = 2.0
point[2] = 3.0
point[3] = 4.0

# ARMACoefficients Point constructor
coefficients3 = ot.ARMACoefficients(point)

# print  of the new collection
print("Using Point constructor ")
print("coefficients3 = ", repr(coefficients3))

# ARMACoefficients with polynomial constructor
point *= 2.0
poly = ot.UniVariatePolynomial(point)

coefficients4 = ot.ARMACoefficients(poly)

# print  of the new collection
print("Using polynomial constructor ")
print("coefficients4 = ", repr(coefficients4))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

matrix1 = ot.CorrelationMatrix(2)
matrix1.setName("matrix1")
print("matrix1 = ", repr(matrix1))

p = ot.Point((1.0, 3.0))
print("matrix1*", p, "=", matrix1 * p)

s = 3.0
print("matrix1*", s, "=", matrix1 * s)
print(s, "*matrix1=", s * matrix1)
print("matrix1/", s, "=", matrix1 / s)

# from SymmetricMatrix
sym = ot.SymmetricMatrix(3)
sym[0, 0] = 1.0e-02
sym[1, 1] = 1.0e-02
sym[2, 2] = 1.0e-02
sym[0, 1] = 7.0e-04
ot.CorrelationMatrix(sym)
print("ok")



"""
Fit a distribution from an input sample
=======================================
"""

# %%
#
# In this example we show how to use the
# :meth:`~openturns.MetaModelAlgorithm.BuildDistribution` function to fit a distribution to an
# input sample.
# This function is used by the :class:`~openturns.FunctionalChaosAlgorithm`
# class when we want to create a polynomial chaos metamodel and we have a
# design of experiments which have been computed beforehand.
# In this case, we have to identify the distributions which best fit to the
# input sample in order to define the input probabilistic model.
# This is, in turn, used by in the polynomial chaos to create the orthogonal basis.
#

# %%
import openturns as ot


# %%
# We first create the function `model`.

# %%
dimension = 2
input_names = ["x1", "x2"]
formulas = ["cos(x1 + x2)", "(x2 + 1) * exp(x1)"]
model = ot.SymbolicFunction(input_names, formulas)

# %%
# Then we create a sample `x` and compute the corresponding output sample `y`.

# %%
distribution = ot.Normal(dimension)
samplesize = 1000
inputSample = distribution.getSample(samplesize)
outputSample = model(inputSample)

# %%
# Create a functional chaos model.
# The algorithm used by :meth:`~openturns.MetaModelAlgorithm.BuildDistribution`
# fits a distribution on the input sample.
# This is done with the Lilliefors test.
# Please read :ref:`kolmogorov_smirnov_test` for more details on this topic.

# %%
# The Lilliefors test is based on sampling the distribution of the
# Kolmogorov-Smirnov statistics.
# The sample size corresponding to this algorithm is configured
# by the `"FittingTest-LillieforsMaximumSamplingSize"` :class:`~openturns.ResourceMap`
# key.
# In order to get satisfactory results, the default value of this
# key is relatively large.

# %%
ot.ResourceMap.GetAsUnsignedInteger("FittingTest-LillieforsMaximumSamplingSize")

# %%
# In order to speed this example up, let us reduce this value.

# %%
ot.ResourceMap.SetAsUnsignedInteger("FittingTest-LillieforsMaximumSamplingSize", 100)

# %%
# Then we fit the distribution.
distribution = ot.FunctionalChaosAlgorithm.BuildDistribution(inputSample)

# %%
# Let us explore the distribution with its fitted parameters.

# %%
distribution

# %%
# We can also analyze its properties in more details.

# %%
for i in range(dimension):
    marginal = distribution.getMarginal(i)
    marginalname = marginal.getImplementation().getClassName()
    print("Marginal #", i, ":", marginalname)
distribution.getCopula()

# %%
# The previous call to :meth:`~openturns.MetaModelAlgorithm.BuildDistribution` is what is done internally by the
# following constructor of :class:`~openturns.FunctionalChaosAlgorithm`.

# %%
algo = ot.FunctionalChaosAlgorithm(inputSample, outputSample)

# %%
# The previous constructor is the main topic of the example
# :doc:`Create a polynomial chaos metamodel from a data set </auto_surrogate_modeling/polynomial_chaos/plot_functional_chaos>`.



import openturns as ot
import openturns.viewer as otv


sample = ot.Normal(3).getSample(100)
distribution = ot.JointDistribution(
    [ot.HistogramFactory().build(sample.getMarginal(i)) for i in range(3)]
)
graph = ot.VisualTest.DrawPairsMarginals(sample, distribution)

otv.View(graph, figure_kw={"figsize": (6.0, 6.0)})



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.TESTPREAMBLE()

# TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER
print("test number zero : default constructor and string converter")

# Default constructor
symmetricMatrix0 = ot.SymmetricMatrix()

# String converter
print("symmetricMatrix0 = ", repr(symmetricMatrix0))

# TEST NUMBER ONE : CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
print("test number one : constructor with size, operator() and string converter")

# Constructor with size
symmetricMatrix1 = ot.SymmetricMatrix(2)

# Check operator() methods
symmetricMatrix1[0, 0] = 1.0
symmetricMatrix1[1, 0] = 2.0
symmetricMatrix1[0, 1] = 3.0
symmetricMatrix1[1, 1] = 4.0

# String converter
print("symmetricMatrix1 = ", repr(symmetricMatrix1))

# TEST NUMBER TWO : COPY CONSTRUCTOR AND STRING CONVERTER
print("test number two : copy constructor and string converter")

# Copy constructor
symmetricMatrix2 = ot.SymmetricMatrix(symmetricMatrix1)

# String converter
print("symmetricMatrix2 = ", repr(symmetricMatrix2))

# TEST NUMBER THREE : GET DIMENSIONS METHODS
print("test number three : get dimensions methods")

# Get dimension methods
print("symmetricMatrix1's nbRows = ", symmetricMatrix1.getNbRows())
print("symmetricMatrix1's nbColumns = ", symmetricMatrix1.getNbColumns())

# TEST NUMBER FIVE : ASSIGNMENT METHOD
print("test number five : assignment method")

# Assignment method
# No sense with python

# TEST NUMBER SIX : TRANSPOSITION METHOD
print("test number six : transposition method")

# Check transpose method
symmetricMatrix4 = symmetricMatrix1.transpose()
print("symmetricMatrix1 transposed = ", repr(symmetricMatrix4))

# TEST NUMBER SEVEN : ADDITION METHOD
print("test number seven : addition method")

# Check addition method : we check the operator and the symmetry of the
# operator, thus testing the comparison operator
sum1 = symmetricMatrix1 + symmetricMatrix4
sum2 = symmetricMatrix4 + symmetricMatrix1
print("sum1 = ", repr(sum1))
print("sum2 = ", repr(sum2))
print("sum1 equals sum2 = ", sum1 == sum2)

# TEST NUMBER EIGHT : SUBTRACTION METHOD
print("test number eight : subtraction method")

# Check subtraction method
diff = symmetricMatrix1 - symmetricMatrix4
print("diff = ", repr(diff))

# TEST NUMBER NINE : MATRIX MULTIPLICATION METHOD
print("test number nine : matrix multiplication method")

# Check multiplication method
prod = symmetricMatrix1 * symmetricMatrix4
print("prod = ", repr(prod))

# TEST NUMBER TEN : MULTIPLICATION WITH A NUMERICAL POINT METHOD
print("test number ten : multiplication with a numerical point method")

# Create the numerical point
pt = ot.Point()
pt.add(1.0)
pt.add(2.0)
print("pt = ", repr(pt))

# Check the product method
ptResult = symmetricMatrix1 * pt
print("ptResult = ", repr(ptResult))

# TEST NUMBER ELEVEN : MULTIPLICATION AND DIVISION BY A NUMERICAL SCALAR
# METHODS
print("test number eleven : multiplication and division by a numerical scalar methods")

# Check the multiplication method
s = 3.0
scalprod1 = symmetricMatrix1 * s
# bug PYTHON scalprod2 = s * symmetricMatrix1
scalprod3 = symmetricMatrix1 * s
print("scalprod1 = ", repr(scalprod1))
# print  "scalprod2 = " , scalprod2
print("scalprod3 = ", repr(scalprod3))
# print  "scalprod1 equals scalprod2 = " , (scalprod1 == scalprod2)
print("scalprod1 equals scalprod3 = ", (scalprod1 == scalprod3))
# print  "scalprod2 equals scalprod3 = " , (scalprod2 == scalprod3)

# Check the division method
scaldiv1 = symmetricMatrix1 / s
scaldiv2 = symmetricMatrix1 / s
print("scaldiv1 = ", repr(scaldiv1))
print("scaldiv2 = ", repr(scaldiv2))
print("scaldiv1 equals scaldiv2 = ", (scaldiv1 == scaldiv2))

# TEST NUMBER TWELVE : ISEMPTY METHOD
print("test number twelve : isEmpty method")

# Check method isEmpty
symmetricMatrix5 = ot.SymmetricMatrix()
symmetricMatrix6 = ot.SymmetricMatrix()
print("symmetricMatrix0 is empty = ", symmetricMatrix0.isEmpty())
print("symmetricMatrix1 is empty = ", symmetricMatrix1.isEmpty())
print("symmetricMatrix5 is empty = ", symmetricMatrix5.isEmpty())

# Check inverse()
symmetricMatrix6 = ot.SymmetricMatrix(
    [[4.0, 2.0, 1.0], [2.0, 5.0, 3.0], [1.0, 3.0, 6.0]]
)
symmetricMatrix7 = symmetricMatrix6.inverse()
inverseReference = ot.SymmetricMatrix(
    [[21.0, -9.0, 1.0], [-9.0, 23.0, -10.0], [1.0, -10.0, 16.0]]
)
inverseReference /= 67.0
assert_almost_equal(symmetricMatrix7, inverseReference)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Big test case for correlated components

# Instantiate one distribution object
dim = 4
meanPoint = ot.Point(dim, 1.0)
sigma = ot.Point(dim, 1.0)
R = ot.CorrelationMatrix(dim)
for i in range(1, dim):
    R[i, i - 1] = 0.5

distribution = ot.Normal(meanPoint, sigma, R)

# Test for sampling
size = 1000
oneSample = distribution.getSample(size)
print(
    "sample of size ",
    size,
    " first=",
    repr(oneSample[0]),
    " last=",
    repr(oneSample[oneSample.getSize() - 1]),
)
mean = oneSample.computeMean()
print("mean error=%.6f" % ((mean - meanPoint).norm() / meanPoint.norm()))
covariance = oneSample.computeCovariance()
errorCovariance = 0.0
for i in range(dim):
    for j in range(dim):
        errorCovariance += abs(covariance[i, j] - sigma[i] * sigma[j] * R[i, j])
print("covariance error=%.6f" % (errorCovariance / (dim * dim)))

# Define a point
zero = ot.Point(dim, 0.0)

# Show PDF and CDF of zero point
zeroPDF = distribution.computePDF(zero)
zeroCDF = distribution.computeCDF(zero)
print(
    "Zero point = ",
    repr(zero),
    " pdf=%.6f" % zeroPDF,
    repr(zero),
    " cdf=%.6f" % zeroCDF,
    " density generator=%.6f" % distribution.computeDensityGenerator(0.0),
)

# Extract the marginals
for i in range(dim):
    margin = distribution.getMarginal(i)
    print("margin=", repr(margin))
    print("margin PDF=%.6f" % margin.computePDF(ot.Point(1)))
    print("margin CDF=%.6f" % margin.computeCDF(ot.Point(1)))
    print("margin quantile=", repr(margin.computeQuantile(0.5)))
    print("margin realization=", repr(margin.getRealization()))

# Extract a 2-D marginal
indices = [1, 0]
print("indices=", repr(indices))
margins = distribution.getMarginal(indices)
print("margins=", repr(margins))
print("margins PDF=%.6f" % margins.computePDF(ot.Point(2)))
print("margins CDF=%.6f" % margins.computeCDF(ot.Point(2)))
quantile = ot.Point(margins.computeQuantile(0.5))
print("margins quantile=", repr(quantile))
print("margins CDF(qantile)=%.6f" % margins.computeCDF(quantile))
print("margins realization=", repr(margins.getRealization()))

# Very big test case for independent components
dim = 200
meanPoint = ot.Point(dim, 0.1)
sigma = ot.Point(dim, 1.0)
distribution = ot.Normal(meanPoint, sigma, ot.IdentityMatrix(dim))
print("Has independent copula? ", distribution.hasIndependentCopula())

# Test for sampling
oneSample = distribution.getSample(size // 10)
print(
    "sample of size ",
    size,
    " first=",
    repr(oneSample[0]),
    " last=",
    repr(oneSample[oneSample.getSize() - 1]),
)
mean = oneSample.computeMean()
print("mean error=%.6f" % ((mean - meanPoint).norm() / meanPoint.norm()))
covariance = oneSample.computeCovariance()
errorCovariance = 0.0
for i in range(dim):
    for j in range(dim):
        if i == j:
            temp = sigma[i] * sigma[j]
        else:
            temp = 0.0
    errorCovariance += abs(covariance[i, j] - temp)
print("covariance error=%.6f" % (errorCovariance / (dim * dim)))

# Define a point
zero = ot.Point(dim, 0.0)

# Show PDF and CDF of zero point
zeroPDF = distribution.computePDF(zero)
zeroCDF = distribution.computeCDF(zero)
print(
    "Zero point= ",
    repr(zero),
    " pdf=%.6f" % zeroPDF,
    " cdf=%.6f" % zeroCDF,
    " density generator=%.6f" % distribution.computeDensityGenerator(0.0),
)



import openturns as ot
import openturns.testing as ott


def compute_length(mesh2D):
    length = 0.0
    for ind in mesh2D.getSimplices():
        length += (mesh2D.getVertex(ind[0]) - mesh2D.getVertex(ind[1])).norm()
    return length


# Define the vertices of the mesh
vertices = [[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [1.5, 1.0], [3.0, 1.5], [0.5, 1.5]]
# Define the simplices of the mesh
simplices = [[0, 1, 2], [1, 2, 3], [2, 3, 4], [2, 4, 5], [0, 2, 5]]
# Create the Mesh
mesh2D = ot.Mesh(vertices, simplices)
# Build the Mesh boundary
mesh2DBoundary = ot.BoundaryMesher().build(mesh2D)
volume = mesh2DBoundary.getVolume()
ott.assert_almost_equal(volume, 7.780311648918275)
length = compute_length(mesh2DBoundary)
ott.assert_almost_equal(length, 7.780311648918275)
# Build a thick outside Mesh boundary
mesh2DBoundaryOutside = ot.BoundaryMesher().build(mesh2D, 0.05)
volume = mesh2DBoundaryOutside.getVolume()
ott.assert_almost_equal(volume, 0.194508)
# Build a thick inside Mesh boundary
mesh2DBoundaryInside = ot.BoundaryMesher().build(mesh2D, -0.05)
volume = mesh2DBoundaryInside.getVolume()
ott.assert_almost_equal(volume, 0.194508)



"""
Create a composed function
==========================
"""

# %%
# In this example we are going to create a composed function :math:`f\circ g`
#

# %%
import openturns as ot


# %%
# assume f, g functions
g = ot.SymbolicFunction(["x1", "x2"], ["x1 + x2", "3 * x1 * x2"])
f = ot.SymbolicFunction(["x1", "x2"], ["2 * x1 - x2"])

# %%
# create the composed function
function = ot.ComposedFunction(f, g)

# %%
# evaluate the function
x = [3.0, 4.0]
y = function(x)
print("x=", x, "y=", y)



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal
from openturns.usecases import cantilever_beam


ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)

"""
Test the import of the CantileverBeam data class.
"""
cb = cantilever_beam.CantileverBeam()

# test parameters
assert_almost_equal(cb.dim, 4, 1e-12)
assert_almost_equal(cb.R[2, 3], -0.2, 1e-12)

# test marginals means
assert_almost_equal(cb.E.getMean()[0], 67045454545.454544, 1e-12)
assert_almost_equal(cb.F.getMean()[0], 299.99999999999994, 1e-12)
assert_almost_equal(cb.II.getMean()[0], 1.4538461538461538e-07, 1e-12)
assert_almost_equal(cb.L.getMean()[0], 2.55, 1e-12)

# special value
X = ot.Point([1.0, 3.0, 1.0, 1.0])
assert_almost_equal(cb.model(X), [1.0], 1e-12)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott


# Test 1
def test_one_input_one_output():
    sampleSize = 6
    dimension = 1

    f = ot.SymbolicFunction(["x0"], ["x0 * sin(x0)"])

    X = ot.Sample(sampleSize, dimension)
    for i in range(sampleSize):
        X[i, 0] = 3.0 + i
    X[0, 0] = 1.0
    X[1, 0] = 3.0
    Y = f(X)

    # create covariance model
    basis = ot.ConstantBasisFactory(dimension).build()
    covarianceModel = ot.SquaredExponential()

    # create algorithm
    fit_algo = ot.GaussianProcessFitter(X, Y, covarianceModel, basis)

    # set sensible optimization bounds and estimate hyper parameters
    fit_algo.setOptimizationBounds(ot.Interval(X.getMin(), X.getMax()))
    fit_algo.setOptimizationAlgorithm(ot.NLopt("LN_NELDERMEAD"))
    fit_algo.run()

    # perform an evaluation
    fit_result = fit_algo.getResult()

    algo = ot.GaussianProcessRegression(fit_result)
    algo.run()
    result = algo.getResult()
    ott.assert_almost_equal(result.getMetaModel()(X), Y)

    # Prediction accuracy
    ott.assert_almost_equal(result.getMetaModel()([7.5]), f([7.5]), 0.3, 0.0)


# Test 2
def test_two_inputs_one_output():

    inputDimension = 2
    # Learning data
    levels = [8, 5]
    box = ot.Box(levels)
    inputSample = box.generate()
    # Scale each direction
    inputSample *= 10.0

    model = ot.SymbolicFunction(["x", "y"], ["cos(0.5*x) + sin(y)"])
    outputSample = model(inputSample)

    # Validation
    sampleSize = 10
    inputValidSample = ot.JointDistribution(2 * [ot.Uniform(0, 10.0)]).getSample(
        sampleSize
    )
    outputValidSample = model(inputValidSample)

    # 2) Definition of exponential model
    # The parameters have been calibrated using TNC optimization
    # and AbsoluteExponential models
    scale = [5.33532, 2.61534]
    amplitude = [1.61536]
    covarianceModel = ot.SquaredExponential(scale, amplitude)

    # 3) Basis definition
    basis = ot.ConstantBasisFactory(inputDimension).build()

    # 4) GPF algorithm
    fit_algo = ot.GaussianProcessFitter(
        inputSample, outputSample, covarianceModel, basis
    )
    # set sensible optimization bounds and estimate hyper parameters
    fit_algo.setOptimizationBounds(
        ot.Interval(inputSample.getMin(), inputSample.getMax())
    )
    fit_algo.setOptimizationAlgorithm(ot.NLopt("LN_NELDERMEAD"))
    fit_algo.run()

    # perform an evaluation
    fit_result = fit_algo.getResult()
    # Regression algorithm
    algo = ot.GaussianProcessRegression(fit_result)
    algo.run()

    result = algo.getResult()
    # Get meta model
    metaModel = result.getMetaModel()
    outData = metaModel(inputValidSample)

    # 5) Errors
    # Interpolation
    ott.assert_almost_equal(outputSample, metaModel(inputSample), 3.0e-5, 3.0e-5)

    # Prediction
    ott.assert_almost_equal(outputValidSample, outData, 1.0e-1, 1e-1)


def test_two_outputs():
    f = ot.SymbolicFunction(["x"], ["x * sin(x)", "x * cos(x)"])
    sampleX = ot.Sample([[1.0], [2.0], [3.0], [4.0], [5.0], [6.0], [7.0], [8.0]])
    sampleY = f(sampleX)
    # Build a basis phi from R --> R^2
    # phi_{0,0} = phi_{0,1} = x
    # phi_{1,0} = phi_{1,1} = x^2
    phi0 = ot.AggregatedFunction(
        [ot.SymbolicFunction(["x"], ["x"]), ot.SymbolicFunction(["x"], ["x"])]
    )
    phi1 = ot.AggregatedFunction(
        [ot.SymbolicFunction(["x"], ["x^2"]), ot.SymbolicFunction(["x"], ["x^2"])]
    )
    basis = ot.Basis([phi0, phi1])
    covarianceModel = ot.SquaredExponential([1.0])
    covarianceModel.setActiveParameter([])
    covarianceModel = ot.TensorizedCovarianceModel([covarianceModel] * 2)

    fit_algo = ot.GaussianProcessFitter(sampleX, sampleY, covarianceModel, basis)
    # set sensible optimization bounds and estimate hyper parameters
    fit_algo.setOptimizationAlgorithm(ot.NLopt("LN_NELDERMEAD"))
    fit_algo.run()

    # perform an evaluation
    fit_result = fit_algo.getResult()
    algo = ot.GaussianProcessRegression(fit_result)
    algo.run()
    result = algo.getResult()
    mm = result.getMetaModel()
    assert mm.getOutputDimension() == 2, "wrong output dim"
    y = mm([5.5])
    print(y)
    ott.assert_almost_equal(y, [-3.88371, 3.90286])


def test_stationary_fun():
    # fix https://github.com/openturns/openturns/issues/1861
    ot.RandomGenerator.SetSeed(0)
    rho = ot.SymbolicFunction("tau", "exp(-abs(tau))*cos(2*pi_*abs(tau))")
    model = ot.StationaryFunctionalCovarianceModel([1], [1], rho)
    x = ot.Normal().getSample(20)
    x.setDescription(["J0"])
    y = x + ot.Normal(0, 0.1).getSample(20)
    y.setDescription(["G0"])

    fit_algo = ot.GaussianProcessFitter(x, y, model, ot.LinearBasisFactory().build())
    # set sensible optimization bounds and estimate hyper parameters
    fit_algo.setOptimizationAlgorithm(ot.NLopt("LN_NELDERMEAD"))
    fit_algo.run()

    # perform an evaluation
    fit_result = fit_algo.getResult()
    algo = ot.GaussianProcessRegression(fit_result)

    algo.run()
    result = algo.getResult()
    mm = result.getMetaModel()
    y = mm([5.5])
    print(y)
    ott.assert_almost_equal(y, [5.58849])


if __name__ == "__main__":
    test_one_input_one_output()
    test_two_inputs_one_output()
    test_two_outputs()
    test_stationary_fun()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

dim = 2
distribution = ot.Normal(dim)


X = ot.RandomVector(distribution)


# 1. Composite/Composite
f1 = ot.SymbolicFunction(["x" + str(i) for i in range(dim)], ["x0"])
f2 = ot.SymbolicFunction(["x" + str(i) for i in range(dim)], ["x1"])

Y1 = ot.CompositeRandomVector(f1, X)
l2 = ot.LevelSet(f2, ot.Greater(), 0.0)

e1 = ot.ThresholdEvent(Y1, ot.Less(), 0.0)
e2 = ot.DomainEvent(X, l2)

e3 = e1.intersect(e2)
# print('e3=', e3)

proba_e3 = e3.getSample(10000).computeMean()[0]
print("proba_e3 = %.3g" % proba_e3)
ott.assert_almost_equal(proba_e3, 0.25, 1e-2, 1e-2)

e4 = e1.join(e2)
# print('e4=', e4)

proba_e4 = e4.getSample(10000).computeMean()[0]
print("proba_e4 = %.3g" % proba_e4)
ott.assert_almost_equal(proba_e4, 0.75, 1e-2, 1e-2)

e5 = ot.IntersectionEvent([e1, e2])
# print('e5=', e5)

proba_e5 = e5.getSample(10000).computeMean()[0]
print("proba_e5 = %.3g" % proba_e5)
ott.assert_almost_equal(proba_e5, 0.25, 1e-2, 1e-2)

e6 = ot.UnionEvent([e1, e2])
# print('e6=', e6)

proba_e6 = e6.getSample(10000).computeMean()[0]
print("proba_e6 = %.3g" % proba_e6)
ott.assert_almost_equal(proba_e6, 0.75, 1e-2, 1e-2)

# intersection of intersection
e7 = ot.IntersectionEvent([e1, ot.IntersectionEvent([e1, e2])])
ott.assert_almost_equal(e7.getSample(10000).computeMean()[0], 0.25, 1e-2, 1e-2)

e8 = ot.IntersectionEvent([ot.IntersectionEvent([e1, e2]), e1])
ott.assert_almost_equal(e8.getSample(10000).computeMean()[0], 0.25, 1e-2, 1e-2)

# union of union
e9 = ot.UnionEvent([e1, ot.UnionEvent([e1, e2])])
ott.assert_almost_equal(e9.getSample(10000).computeMean()[0], 0.75, 1e-2, 1e-2)

e10 = ot.UnionEvent([ot.UnionEvent([e1, e2]), e1])
ott.assert_almost_equal(e10.getSample(10000).computeMean()[0], 0.75, 1e-2, 1e-2)

# intersection of union
e11 = ot.IntersectionEvent([ot.UnionEvent([e1, e2]), ot.UnionEvent([e1, e2])])
ott.assert_almost_equal(e11.getSample(10000).computeMean()[0], 0.75, 1e-2, 1e-2)

# union of intersection
e12 = ot.UnionEvent([ot.IntersectionEvent([e1, e2]), ot.IntersectionEvent([e1, e2])])
ott.assert_almost_equal(e12.getSample(10000).computeMean()[0], 0.25, 1e-2, 1e-2)

# DomainEvent
domain = ot.Interval([-1.0] * dim, [1.0] * dim)
e13 = ot.DomainEvent(X, domain)
e13_probability = distribution.computeProbability(domain)
ott.assert_almost_equal(
    e13.getSample(10000).computeMean()[0], e13_probability, 1e-2, 1e-2
)

# Union with DomainEvent
e14 = ot.UnionEvent([e1, e2, e13])
ott.assert_almost_equal(
    e14.getSample(10000).computeMean()[0], 0.75 + e13_probability / 4, 1e-2, 1e-2
)

# Intersection with DomainEvent
e15 = ot.IntersectionEvent([e1, e2, e13])
ott.assert_almost_equal(
    e15.getSample(10000).computeMean()[0], e13_probability / 4, 1e-2, 1e-2
)

# through simulation


def sim_event(ev):
    experiment = ot.MonteCarloExperiment()
    algo = ot.ProbabilitySimulationAlgorithm(ev, experiment)
    algo.setMaximumOuterSampling(2500)
    algo.setBlockSize(4)
    algo.setMaximumCoefficientOfVariation(-1.0)
    algo.run()
    result = algo.getResult()
    return result.getProbabilityEstimate()


ott.assert_almost_equal(sim_event(e5), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e6), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e7), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e8), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e9), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e10), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e11), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e12), 0.25, 1e-2, 2e-2)
ott.assert_almost_equal(sim_event(e13), e13_probability, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e14), 0.75 + e13_probability / 4, 1e-2, 1e-2)
ott.assert_almost_equal(sim_event(e15), e13_probability / 4, 1e-2, 1e-2)


def subset_event(ev):
    algo = ot.SubsetSampling(ev)
    algo.setMaximumOuterSampling(2500)
    algo.setBlockSize(4)
    algo.run()
    result = algo.getResult()
    return result.getProbabilityEstimate()


ott.assert_almost_equal(subset_event(e3), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e5), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e6), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e7), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e8), 0.25, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e9), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e10), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e11), 0.75, 1e-2, 1e-2)
ott.assert_almost_equal(subset_event(e12), 0.25, 1e-2, 1e-2)


# check that f2 is not called when not needed
f1 = ot.SymbolicFunction(["x" + str(i) for i in range(dim)], ["x0"])
f2 = ot.MemoizeFunction(ot.SymbolicFunction(["x" + str(i) for i in range(dim)], ["x1"]))
Y1 = ot.CompositeRandomVector(f1, X)
Y2 = ot.CompositeRandomVector(f2, X)
e1 = ot.ThresholdEvent(Y1, ot.Less(), 1e6)
e2 = ot.ThresholdEvent(Y2, ot.Less(), 0.0)
union = ot.UnionEvent([e1, e2])
proba = union.getSample(1000).computeMean()[0]
print(proba)
assert proba == 1.0, "always true"
assert f2.getCallsNumber() == 0, "union prune"
e1 = ot.ThresholdEvent(Y1, ot.Greater(), 1e6)
e2 = ot.ThresholdEvent(Y2, ot.Less(), 0.0)
intersection = ot.IntersectionEvent([e1, e2])
proba = intersection.getSample(1000).computeMean()[0]
print(proba)
assert proba == 0.0, "always false"
assert f2.getCallsNumber() == 0, "intersection prune"


# check batch evaluation
n = 10000


def f1py(x):
    x0 = ot.Sample(x).getMarginal(0)
    print("eval f1")
    assert x0.getSize() == n, "no batch eval"
    return x0


def f2py(x):
    x1 = ot.Sample(x).getMarginal(1)
    print("eval f2")
    assert x1.getSize() > 1, "no batch eval"
    return x1


f1 = ot.PythonFunction(2, 1, func_sample=f1py)
f2 = ot.PythonFunction(2, 1, func_sample=f2py)
Y1 = ot.CompositeRandomVector(f1, X)
Y2 = ot.CompositeRandomVector(f2, X)
e1 = ot.ThresholdEvent(Y1, ot.Less(), 0.0)
e2 = ot.ThresholdEvent(Y2, ot.Greater(), 0.0)
e3 = ot.UnionEvent([e1, e2])
p3 = e3.getSample(n).computeMean()[0]
ott.assert_almost_equal(p3, 0.75, 1e-2, 1e-2)
e4 = ot.IntersectionEvent([e1, e2])
p4 = e4.getSample(n).computeMean()[0]
ott.assert_almost_equal(p4, 0.25, 1e-2, 1e-2)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
# ot.Log.Show(ot.Log.ALL)

dim = 2

# problem
model = ot.SymbolicFunction(
    ["x", "y"],
    ["3*(1-x)^2*exp(-x^2-(y+1)^2)-10*(x/5-x^3-y^5)*exp(-x^2-y^2)-exp(-(x+1)^2-y^2)/3"],
)
bounds = ot.Interval([-3.0] * dim, [3.0] * dim)
problem = ot.OptimizationProblem(model)
problem.setBounds(bounds)

# solver
solver = ot.TNC(problem)

# run locally
solver.setStartingPoint([0.0] * dim)
algo = solver
algo.run()
result = algo.getResult()
local_optimal_point = [0.296446, 0.320196]
local_optimal_value = [-0.0649359]
ott.assert_almost_equal(result.getOptimalPoint(), local_optimal_point, 1e-5, 0.0)
ott.assert_almost_equal(result.getOptimalValue(), local_optimal_value, 1e-5, 0.0)


# multistart
lower_bound = bounds.getLowerBound()
upper_bound = bounds.getUpperBound()
dist_collection = []
for i in range(dim):
    dist_collection.append(ot.Uniform(lower_bound[i], upper_bound[i]))
distribution = ot.JointDistribution(dist_collection)
size = 20
experiment = ot.LHSExperiment(distribution, size)
startingSample = experiment.generate()
algo = ot.MultiStart(solver, startingSample)
algo.setMaximumCallsNumber(2000)
algo.run()
result = algo.getResult()
true_optimal_point = [0.228279, -1.62553]
true_optimal_value = [-6.55113]
ott.assert_almost_equal(result.getOptimalPoint(), true_optimal_point, 1e-5, 0.0)
ott.assert_almost_equal(result.getOptimalValue(), true_optimal_value, 1e-5, 0.0)
n_local_results = algo.getResultCollection().getSize()
assert n_local_results == 20, "n_local_results is wrong"
algo.setKeepResults(False)
algo.run()
n_local_results_nokeep = algo.getResultCollection().getSize()
assert n_local_results_nokeep == 0, "n_local_results_nokeep is wrong"
assert result.getCallsNumber() < 400, "wrong calls number"



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

myDist = [ot.Beta(1.5, 1.7, 0.0, 1.0), ot.Beta(2.0, 2.3, 0.5, 1.2)]
myOrderStatDist = ot.MaximumEntropyOrderStatisticsDistribution(myDist)
myOrderStatDist.setDescription(["$x_1$", "$x_2$"])
graphPDF = myOrderStatDist.drawPDF()
graphCDF = myOrderStatDist.drawCDF()


fig = plt.figure(figsize=(8, 4))
pdf_axis = fig.add_subplot(121)
cdf_axis = fig.add_subplot(122)
pdf_axis.set_xlim(auto=True)
cdf_axis.set_xlim(auto=True)

otv.View(graphPDF, figure=fig, axes=[pdf_axis], add_legend=True)
otv.View(graphCDF, figure=fig, axes=[cdf_axis], add_legend=True)
fig.suptitle("Max Entropy Order Statistics Distribution: pdf and cdf")



"""
Estimate tail dependence coefficients on the wave-surge data
============================================================
"""

# %%
# In this example we estimate the tail dependence coefficient of a bivariate sample
# applied to the concurrent measurements of two oceanographic variables (wave and surge heights) at a single location off south-west England.
# Readers should refer to [coles2001]_ to get more details.
#
# First, we load the wave-surge dataset.
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import coles

data = coles.Coles().wavesurge
print(data[:5])

graph = ot.Graph("Concurrent wave and surge heights", "wave (m)", "surge (m)")
cloud = ot.Cloud(data)
cloud.setColor("red")
graph.add(cloud)
view = otv.View(graph)

# %%
# We plot the graph of the function :math:`u \mapsto \chi(u)` and the graph of the function
# :math:`u \mapsto \bar{\chi}(u)`. We conclude that both variables are asymptotially dependent
# as :math:`\chi > 0` and that they are positively correlated as :math:`\hat{\chi} > 0`.
# We can visually deduce the upper tail dependence coefficient :math:`\chi \simeq 0.26` and
# the upper extremal dependence coefficient :math:`\bar{\chi} \simeq 0.5`.
graph1 = ot.VisualTest.DrawUpperTailDependenceFunction(data)
graph2 = ot.VisualTest.DrawUpperExtremalDependenceFunction(data)
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, graph1)
grid.setGraph(0, 1, graph2)
view = otv.View(grid)


# %%
otv.View.ShowAll()



"""
Create a random vector
======================
"""

# %%
# The :class:`~openturns.RandomVector` object represents the concept of random variable.
#
# This class can be cerated by:
#
# - Case 1: directly using the :class:`~openturns.RandomVector` class,
# - Case 2: from python, using the :class:`~openturns.PythonRandomVector` class which enables to
#   redefine some methods (as the sampling method for example).

# %%
import openturns as ot

# %%
# Case 1: use the :class:`~openturns.RandomVector` class
# ------------------------------------------------------
#
# We create the random vector :math:`(X_0, X_1, X_2)` following the Normal
# distribution with zero mean and unit variance.
dist3d = ot.Normal(3)
X = ot.RandomVector(dist3d)

# %%
# Get the dimension
X.getDimension()

# %%
# Get the mean
X.getMean()

# %%
# Get the covariance
X.getCovariance()

# %%
# Draw a sample
X.getSample(5)

# %%
# Extract the component :math:`X_1`:
X1 = X.getMarginal(1)
X1.getSample(5)

# %%
# Extract the component :math:`(X_1, X_2)`.
X02 = X.getMarginal([0, 2])
X02.getSample(5)

# %%
# Case 2: use the :class:`~openturns.PythonRandomVector` class
# ------------------------------------------------------------
#
# We create a random vector using the :class:`~openturns.PythonRandomVector` class, which
# enables to overload the following methods: *getRealization*, *getSample*,
# *getMean* and *getCovariance*.
#
# Inherit :class:`~openturns.PythonRandomVector`
# Here, we create a random vector of dimension 2 :math:`(X_1, X_2)`, where :math:`X_1 \sim \cU(0,1)` and
# :math:`X_1 \sim \cU(1,3)` with independent components.


class RVEC(ot.PythonRandomVector):
    def __init__(self):
        super(RVEC, self).__init__(2)
        self.setDescription(["R", "S"])

    def getRealization(self):
        X = [ot.RandomGenerator.Generate(), 2.0 + ot.RandomGenerator.Generate()]
        return X

    def getSample(self, size):
        X = []
        for i in range(size):
            X.append(
                [ot.RandomGenerator.Generate(), 2.0 + ot.RandomGenerator.Generate()]
            )
        return X

    def getMean(self):
        return [0.5, 2.5]

    def getCovariance(self):
        return [[1.0 / 12.0, 0.0], [0.0, 1.0 / 12.0]]


# %%
# Instantiate the distribution
randomVector = ot.RandomVector(RVEC())

# %%
# Get a sample
randomVector.getSample(5)

# %%
# Get its mean
randomVector.getMean()

# %%
# Compute the probability contained in an interval
randomVector.getCovariance()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
if not ot.PlatformInfo.HasFeature("boost"):
    exit(0)

dim = 2
nu = 4.5
R = ot.CorrelationMatrix(dim, [1.0, 0.5, 0.5, 1.0])
distribution = ot.StudentCopula(nu, R)
size = 100000
sample = distribution.getSample(size)

factory = ot.StudentCopulaFactory()
estimatedDistribution = factory.build(sample)
print("distribution=", repr(distribution))
print("Estimated distribution=", repr(estimatedDistribution))
ott.assert_almost_equal(estimatedDistribution.getParameter()[0], nu, 1e-1, 1e-1)
ott.assert_almost_equal(estimatedDistribution.getParameter()[-1], 0.5, 1e-1, 1e-1)
estimatedDistribution = factory.build()
print("Default distribution=", estimatedDistribution)
estimatedDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters=", estimatedDistribution)
estimatedStudentCopula = factory.buildAsStudentCopula(sample)
print("StudentCopula          =", distribution)
print("Estimated copula=", estimatedStudentCopula)
estimatedStudentCopula = factory.buildAsStudentCopula()
print("Default copula=", estimatedStudentCopula)
estimatedStudentCopula = factory.buildAsStudentCopula(distribution.getParameter())
print("StudentCopula from parameters=", estimatedStudentCopula)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
dim = 10
R = ot.CorrelationMatrix(dim)
for i in range(dim):
    for j in range(i):
        R[i, j] = (i + j + 1.0) / (2.0 * dim)
mean = [2.0] * dim
sigma = [3.0] * dim
distribution = ot.Normal(mean, sigma, R)

size = 100
sample = distribution.getSample(size)
sampleY = sample.getMarginal(0)

sampleZ = ot.Sample(size, 1)
for i in range(size):
    sampleZ[i, 0] = sampleY[i, 0] * sampleY[i, 0]

discreteSample1 = ot.Poisson(0.1).getSample(size)
discreteSample2 = ot.Geometric(0.4).getSample(size)

# ChiSquared Independence test : test if two samples (of sizes not necessarily equal) are independent ?
# Care : discrete samples only
# H0 = independent samples
# p-value threshold : probability of the H0 reject zone : 0.10
# p-value : probability (test variable decision > test variable decision evaluated on the samples)
# Test = True <=> p-value > p-value threshold
print(
    "ChiSquared=",
    ot.HypothesisTest.ChiSquared(discreteSample1, discreteSample2, 0.10),
)
print(
    "ChiSquared2=",
    ot.HypothesisTest.ChiSquared(discreteSample1, discreteSample1, 0.10),
)

# Pearson Test : test if two gaussian samples are independent (based on the evaluation of the linear correlation coefficient)
# H0 : independent samples (linear correlation coefficient = 0)
# Test = True <=> independent samples (linear correlation coefficient = 0)
# p-value threshold : probability of the H0 reject zone : 0.10
# p-value : probability (test variable decision > test variable decision evaluated on the samples)
# Test = True <=> p-value > p-value threshold
print("Pearson=", ot.HypothesisTest.Pearson(sampleY, sampleZ, 0.10))

ot.RandomGenerator.SetSeed(0)
sample1 = ot.Normal().getSample(20)
sample2 = ot.Normal(0.1, 1.1).getSample(30)
resultTwoSamplesKolmogorov = ot.HypothesisTest.TwoSamplesKolmogorov(sample1, sample2)
print(resultTwoSamplesKolmogorov)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER
print("test number zero : default constructor and string converter")

# Default constructor
type = "testResult"
testBool = True
testVal = 0.372
testThres = 0.42
statistic = 0.59
testResultTest = ot.TestResult(type, testBool, testVal, testThres, statistic)

# String converter
print("testResultTest = ", testResultTest)

# TEST NUMBER ONE : STRING CONVERTER
print("test number one : string converter")

# String converter
print("testResultTest = ", testResultTest)

# TEST NUMBER TWO : ELEMENTS ACCESSORS
print("test number two : elements accessors")
print(
    "testResultTest2 binary quality measure = ",
    testResultTest.getBinaryQualityMeasure(),
)
print("testResultTest2 p value = %.6f" % testResultTest.getPValue())
print("testResultTest2 p value threshold = %.6f" % testResultTest.getThreshold())
print("testResultTest2 test type = ", testResultTest.getTestType())

# TEST NUMBER THREE : EQUALITY
print("test number three : equality")
typeDiff = "testResult2"
testBoolDiff = False
testValDiff = 0.02
testThresDiff = 0.12
statisticDiff = 0.19
testResultTestDiff = ot.TestResult(
    typeDiff, testBoolDiff, testValDiff, testThresDiff, statisticDiff
)
print("testResultTestDiff = testResultTest ", (testResultTestDiff == testResultTest))



"""
Use case : Wing weight test function
====================================
"""

import openturns as ot
import math as m


def functionWingWeight(x):
    Sw = x[0]
    Wfw = x[1]
    A = x[2]
    Lambda = x[3]
    q = x[4]
    ll = x[5]
    tc = x[6]
    Nz = x[7]
    Wdg = x[8]
    Wp = x[9]
    return [
        (
            0.036
            * Sw**0.758
            * Wfw**0.0035
            * (A / m.cos(m.pi / 180 * Lambda) ** 2) ** 0.6
            * q**0.006
            * ll**0.04
            * (100 * tc / m.cos(m.pi / 180 * Lambda)) ** (-0.3)
            * (Nz * Wdg) ** 0.49
            + Sw * Wp
        )
    ]


class WingWeightModel:
    """
    Data class for the Wing weight model.

    Attributes
    ----------

    dim : int
        Dimension of the problem, dim = 10

    Sw : :class:`~openturns.Uniform`
        Wing area (ft^2)  distribution
        First marginal, ot.Uniform(150, 200)

    Wfw : :class:`~openturns.Uniform`
        Weight of fuel in the wing (lb) distribution
        Second marginal, ot.Uniform(220, 300)

    A : :class:`~openturns.Uniform`
        Aspect ratio (-) distribution
        Third marginal, ot.Uniform(6, 10)

    Lambda : :class:`~openturns.Uniform`
        Quarter chord sweep (deg),  distribution
        Fourth marginal, ot.Uniform(-10, 10)

    q : :class:`~openturns.Uniform`
        Dynamic pressure at cruise (lb/ft^2) distribution
        Fifth marginal, ot.Uniform(16, 45 )

    l : :class:`~openturns.Uniform`
        Taper ratio (-) distribution
        Sixth marginal, ot.Uniform(0.5, 1)

    tc : :class:`~openturns.Uniform`
        Airfoil thickness to chord ratio (-) distribution
        Seventh marginal, ot.Uniform(0.08, 0.18)

    Nz : :class:`~openturns.Uniform`
        Ultimate load factor (-) distribution
        Eighth marginal, ot.Uniform(2.5, 6)

    Wdg : :class:`~openturns.Uniform`
        Flight design gross weight (lb) distribution
        Ninth marginal, ot.Uniform(1700, 2500)

    Wp : :class:`~openturns.Uniform`
        Paint weight (lb/ft^2) distribution
        Tenth marginal, ot.Uniform(0.025, 0.08)

    distribution : :class:`~openturns.JointDistribution`
        The joint distribution of the input parameters.

    model : :class:`~openturns.PythonFunction`
        The Wing weight model with Sw, Wfw, A, Lambda, q, l, tc, Nz, Wdg and Wp as variables.

    Examples
    --------
    >>> from openturns.usecases import wingweight_function
    >>> # Load the Wing weight model
    >>> ww = wingweight_function.WingWeightModel()
    """

    def __init__(self):
        # dimension
        self.dim = 10

        # First marginal : Sw
        self.Sw = ot.Uniform(150.0, 200.0)
        self.Sw.setDescription(["Sw"])

        # Second marginal : Wfw
        self.Wfw = ot.Uniform(220.0, 300.0)
        self.Wfw.setDescription(["Wfw"])

        # Third marginal : A
        self.A = ot.Uniform(6.0, 10.0)
        self.A.setDescription(["A"])

        # Fourth marginal : Lambda
        self.Lambda = ot.Uniform(-10.0, 10.0)
        self.Lambda.setDescription(["Lambda"])

        # Fifth marginal : q
        self.q = ot.Uniform(16.0, 45.0)
        self.q.setDescription(["q"])

        # Sixth marginal : l
        self.ll = ot.Uniform(0.5, 1.0)
        self.ll.setDescription(["l"])

        # Seventh marginal : tc
        self.tc = ot.Uniform(0.08, 0.18)
        self.tc.setDescription(["tc"])

        # Eighth marginal : Nz
        self.Nz = ot.Uniform(2.5, 6.0)
        self.Nz.setDescription(["Nz"])

        # Ninth marginal : Wdg
        self.Wdg = ot.Uniform(1700.0, 2500.0)
        self.Wdg.setDescription(["Wdg"])

        # Tenth marginal : Wp
        self.Wp = ot.Uniform(0.025, 0.08)
        self.Wp.setDescription(["Wp"])

        # Input distribution
        self.distribution = ot.JointDistribution(
            [
                self.Sw,
                self.Wfw,
                self.A,
                self.Lambda,
                self.q,
                self.ll,
                self.tc,
                self.Nz,
                self.Wdg,
                self.Wp,
            ]
        )
        self.distribution.setDescription(
            ["Sw", "Wfw", "A", "Lambda", "q", "l", "tc", "Nz", "Wdg", "Wp"]
        )
        self.inputDistribution = self.distribution  # deprecated

        # The Wing weight model
        self.model = ot.PythonFunction(10, 1, functionWingWeight)
        self.model.setInputDescription(self.distribution.getDescription())
        self.model.setOutputDescription(["WingWeight"])



"""
Optimization with constraints
=============================
"""

# %%
# In this example we are going to expose methods to solve a generic optimization problem in the form
#
# .. math::
#     \min_{x\in B} f(x) \\
#        g(x) = 0 \\
#        h(x) \ge 0
#

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# define the objective function
objective = ot.SymbolicFunction(
    ["x1", "x2", "x3", "x4"], ["x1 + 2 * x2 - 3 * x3 + 4 * x4"]
)

# %%
# define the constraints
inequality_constraint = ot.SymbolicFunction(["x1", "x2", "x3", "x4"], ["x1-x3"])

# %%
# define the problem bounds
dim = objective.getInputDimension()
bounds = ot.Interval([-3.0] * dim, [5.0] * dim)

# %%
# define the problem
problem = ot.OptimizationProblem(objective)
problem.setMinimization(True)
problem.setInequalityConstraint(inequality_constraint)
problem.setBounds(bounds)

# %%
# solve the problem
algo = ot.Cobyla()
algo.setProblem(problem)
startingPoint = [0.0] * dim
algo.setStartingPoint(startingPoint)
algo.run()

# %%
# retrieve results
result = algo.getResult()
print("x^=", result.getOptimalPoint())

# %%
# draw optimal value history
graph = result.drawOptimalValueHistory()
view = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Instantiate one distribution object
dim = 2
copula = ot.FarlieGumbelMorgensternCopula(0.7)
copula.setName("a farlieGumbelMorgenstern copula")
print("Copula ", repr(copula))
print("Copula ", copula)
print("Mean ", copula.getMean())
print("Covariance ", copula.getCovariance())
# Is this copula an elliptical distribution?
print("Elliptical distribution= ", copula.isElliptical())

# Is this copula elliptical ?
print("Elliptical copula= ", copula.hasEllipticalCopula())

# Is this copula independent ?
print("Independent copula= ", copula.hasIndependentCopula())

# Test for realization of copula
oneRealization = copula.getRealization()
print("oneRealization=", oneRealization)

# Define a point
point = ot.Point(dim, 0.2)

# Show PDF and CDF of zero point
zeroPDF = copula.computePDF(point)
zeroCDF = copula.computeCDF(point)
print("point= ", point, " pdf=", zeroPDF, " cdf= %.12g" % zeroCDF)

# Get 50% quantile
quantile = copula.computeQuantile(0.5)
print("Quantile=", quantile)
print("CDF(quantile)= %.12g" % copula.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(copula.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print(
    "Survival(inverseSurvival)=%.6f" % copula.computeSurvivalFunction(inverseSurvival)
)
print("entropy=%.6f" % copula.computeEntropy())

# Confidence regions
interval, threshold = copula.computeMinimumVolumeIntervalWithMarginalProbability(0.95)
print("Minimum volume interval=", interval)
print("threshold=", ot.Point(1, threshold))
levelSet, beta = copula.computeMinimumVolumeLevelSetWithThreshold(0.95)
print("Minimum volume level set=", levelSet)
print("beta=", ot.Point(1, beta))
interval, beta = copula.computeBilateralConfidenceIntervalWithMarginalProbability(0.95)
print("Bilateral confidence interval=", interval)
print("beta=", ot.Point(1, beta))
interval, beta = copula.computeUnilateralConfidenceIntervalWithMarginalProbability(
    0.95, False
)
print("Unilateral confidence interval (lower tail)=", interval)
print("beta=", ot.Point(1, beta))
interval, beta = copula.computeUnilateralConfidenceIntervalWithMarginalProbability(
    0.95, True
)
print("Unilateral confidence interval (upper tail)=", interval)
print("beta=", ot.Point(1, beta))

# Extract the marginals
for i in range(dim):
    margin = copula.getMarginal(i)
    print("margin=", margin)
    print("margin PDF=", margin.computePDF(ot.Point(1, 0.25)))
    print("margin CDF=", margin.computeCDF(ot.Point(1, 0.25)))
    print("margin quantile=", margin.computeQuantile(0.95))
    print("margin realization=", margin.getRealization())

# Extract a 2-D marginal
indices = [1, 0]
print("indices=", indices)
margins = copula.getMarginal(indices)
print("margins=", margins)
print("margins PDF=", margins.computePDF(ot.Point(2, 0.25)))
print("margins CDF= %.12g" % margins.computeCDF(ot.Point(2, 0.25)))
quantile = margins.computeQuantile(0.95)
print("margins quantile=", quantile)
print("margins CDF(quantile)= %.12g" % margins.computeCDF(quantile))
print("margins realization=", margins.getRealization())

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(copula)
validation.run()



#! /usr/bin/env python

import openturns as ot

distribution = ot.Gumbel(0.5, 2.5)
size = 10000
sample = distribution.getSample(size)
factory = ot.GumbelFactory()
print("Distribution                      =", repr(distribution))
result = factory.buildEstimator(sample)
estimatedDistribution = result.getDistribution()
print("Estimated distribution            =", repr(estimatedDistribution))
parameterDistribution = result.getParameterDistribution()
print("Parameter distribution            =", parameterDistribution)
defaultDistribution = factory.build()
print("Default distribution              =", defaultDistribution)
fromParameterDistribution = factory.build(distribution.getParameter())
print("Distribution from parameters      =", fromParameterDistribution)
typedEstimatedDistribution = factory.buildAsGumbel(sample)
print("Typed estimated distribution      =", typedEstimatedDistribution)
defaultTypedDistribution = factory.buildAsGumbel()
print("Default typed distribution        =", defaultTypedDistribution)
typedFromParameterDistribution = factory.buildAsGumbel(distribution.getParameter())
print("Typed distribution from parameters=", typedFromParameterDistribution)
result = factory.buildEstimator(sample, ot.GumbelMuSigma())
estimatedDistribution = result.getDistribution()
print("Estimated distribution (mu/sigma)       =", repr(estimatedDistribution))
parameterDistribution = result.getParameterDistribution()
print("Parameter distribution (mu/sigma)       =", parameterDistribution)



"""
Create a Point Conditional Distribution
=======================================
"""

# %%
# In this example, we create distributions conditionned by a particular value of their marginals.
#
# We consider the following cases:
#
# - Case 1:  Bivariate distribution with continuous marginals,
# - Case 2:  Bivariate distribution with continuous and discrete marginals,
# - Case 3:  Trivariate distribution with continuous and discrete marginals,
# - Case 4:  Extreme values conditioning.
#
# We illustrate the fact that the range of the conditioned distribution is updated.

# %%
import openturns as ot
import openturns.viewer as otv

ot.ResourceMap.SetAsString("Contour-DefaultColorMapNorm", "rank")

# %%
# Case 1: Bivariate distribution with continuous marginals
# --------------------------------------------------------
#
# Here, we consider the bivariate distribution of :math:`(X_0, X_1)` defined by:
#
# ===========      ==================================================  ===============================
# Variable         Distribution                                        Parameter
# ===========      ==================================================  ===============================
# :math:`X_0`      :class:`~openturns.Gamma` (:math:`k, \lambda`)      :math:`(k, \lambda) = (5, 0.5)`
# :math:`X_1`      :class:`~openturns.Student` (:math:`\nu`)           :math:`\nu = 5`
# Copula           :class:`~openturns.ClaytonCopula` (:math:`\theta`)  :math:`\theta = 2.5`
# ===========      ==================================================  ===============================
#
# We condition the marginal :math:`X_1` to be equal to its quantiles of order
# 0.05, 0.5 and then 0.95. Then, we draw the resulting conditioned distribution of:
#
# .. math::
#
#    X_0 | X_1 = x_1
#
# We define initial distribution of :math:`(X_0, X_1)`.
coll_marg = [ot.Gamma(5.0, 0.5), ot.Student(5.0)]
cop = ot.ClaytonCopula(2.5)
dist_X = ot.JointDistribution(coll_marg, cop)

# %%
# We draw the PDF of :math:`(X_0, X_1)`.
g_X = dist_X.drawPDF()
g_X.setTitle(r"$(X_0, X_1)$: iso-lines PDF")
g_X.setXTitle(r"$x_0$")
g_X.setYTitle(r"$x_1$")

# %%
# We define the conditioned marginal, the list of the conditioning values and we create
# all the conditioned distributions, whose PDF we draw on the same graph.
#
# We also print the updated range of the distribution of :math:`X_0`.
cond_indices = [1]
q_list = [0.05, 0.5, 0.95]
cond_value_list = [coll_marg[1].computeQuantile(q)[0] for q in q_list]

g_cond = coll_marg[0].drawPDF()
g_cond.setLegends([r"dist of $X_0$"])
for index, cond_value in enumerate(cond_value_list):
    cond_value = cond_value_list[index]
    cond_dist = ot.PointConditionalDistribution(dist_X, cond_indices, [cond_value])
    draw = cond_dist.drawPDF().getDrawable(0)
    draw.setLegend(r"$x_1 = q($" + str(q_list[index]) + ")")
    g_cond.add(draw)
    print(
        "Quantile of the cond value = ", q_list[index], "Range : ", cond_dist.getRange()
    )

g_cond.setTitle(r"PDF of $X_0|X_1 = x_1$")
g_cond.setColors(ot.Drawable.BuildDefaultPalette(len(q_list) + 1))

# %%
# We gather both graphs into one grid.
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, g_X)
grid.setGraph(0, 1, g_cond)
# sphinx_gallery_thumbnail_number =  1
view = otv.View(grid)

# %%
# Case 2: Bivariate distribution with continuous and discrete marginals
# ---------------------------------------------------------------------
#
# Here, we consider the bivariate distribution of :math:`(X_0, X_1)` defined by:
#
# ===========      ==================================================  ===============================
# Variable         Distribution                                        Parameter
# ===========      ==================================================  ===============================
# :math:`X_0`      :class:`~openturns.Binomial` (:math:`n,p`)          :math:`(n,p) = (25, 0.2)`
# :math:`X_1`      :class:`~openturns.Poisson` (:math:`\lambda`)       :math:`\lambda = 1`
# Copula           :class:`~openturns.GumbelCopula` (:math:`\theta`)   :math:`\theta = 2`
# ===========      ==================================================  ===============================
#
# We condition the marginal :math:`X_1` to be equal to its quantiles of order
# 0.05, 0.5 and then 0.95. Then, we draw the resulting conditioned distribution of:
#
# .. math::
#
#    X_0 | X_1 = x_1
#
# We go through the same steps as before.
coll_marg = [ot.Binomial(25, 0.2), ot.Poisson(1.0)]
cop = ot.GumbelCopula(2)
dist_X = ot.JointDistribution(coll_marg, cop)

# %%
# We draw the PDF of :math:`(X_0, X_1)`.
g_X = dist_X.drawPDF()
g_X.setTitle(r"$(X_0, X_1)$: iso-lines PDF")
g_X.setXTitle(r"$x_0$")
g_X.setYTitle(r"$x_1$")

# %%
# We define the conditioned marginal, the list of the conditioning values and we create
# all the conditioned distributions, whose PDF we draw on the same graph. To make them all visible,
# we artificially shifted the discrete distributions by a factor 0.1, 0.2 and 0.3.
#
# We also print the updated range of the distribution of :math:`X_0`.
cond_value_list = [coll_marg[1].computeQuantile(q)[0] for q in q_list]

g_cond = coll_marg[0].drawPDF()
g_cond.setLegends([r"dist of $X_0$"])
for index, cond_value in enumerate(cond_value_list):
    cond_value = cond_value_list[index]
    cond_dist = ot.PointConditionalDistribution(dist_X, cond_indices, [cond_value])
    # Here, we shift the distribution to make it visible!
    draw = ((index / 10.0) + cond_dist).drawPDF().getDrawable(0)
    draw.setLegend(r"$x_1 = q($" + str(q_list[index]) + ")")
    g_cond.add(draw)
    print(
        "Quantile of the cond value = ", q_list[index], "Range : ", cond_dist.getRange()
    )

g_cond.setTitle(r"PDF of $X_0|X_1 = x_1$")
g_cond.setLegendPosition("upper left")
g_cond.setColors(ot.Drawable.BuildDefaultPalette(len(q_list) + 1))

# %%
# We gather both graphs into one grid.
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, g_X)
grid.setGraph(0, 1, g_cond)
view = otv.View(grid)


# %%
# Case 3: Trivariate distribution with continuous and discrete marginals
# ----------------------------------------------------------------------
#
# Here, we consider the trivariate distribution of :math:`(X_0, X_1, X_2)` defined by:
#
# ===========      ==================================================  ===============================
# Variable         Distribution                                        Parameter
# ===========      ==================================================  ===============================
# :math:`X_0`      :class:`~openturns.Normal` (:math:`\mu, \sigma`)    :math:`(\mu, \sigma) = (0,1)`
# :math:`X_1`      :class:`~openturns.Poisson` (:math:`\lambda`)       :math:`\lambda = 1`
# :math:`X_2`      :class:`~openturns.Uniform` (:math:`a,b`)           :math:`(a,b) = (-1, 1)`
# Copula           :class:`~openturns.NormalCopula` (:math:`\mat{R}`)  see below
# ===========      ==================================================  ===============================
#
# where the correlation matrix :math:`\mat{R}` of the normal copula is defined by:
#
# .. math::
#
#   R = \left( \begin{array}{ccc}
#       1 & 0.2 & 0.7 \\
#       0.2 & 1 & -0.5 \\
#       0.7 & -0.5 & 1
#       \end{array} \right)
#
# We condition the marginal :math:`X_1` to be equal to its quantiles of order
# 0.05, 0.5 and then 0.95. Then, we draw the resulting bivariate conditioned distribution of:
#
# .. math::
#
#    (X_0,X_2) | X_1 = x_1
#
# We go through the same steps as before.
coll_marg = [ot.Normal(), ot.Poisson(1.0), ot.Uniform()]
R = ot.CorrelationMatrix(3)
R[0, 1] = 0.2
R[0, 2] = 0.7
R[1, 2] = -0.5
cop = ot.NormalCopula(R)
dist_X = ot.JointDistribution(coll_marg, cop)

# %%
# We draw the distribution of :math:`(X_0, X_2)`.
g_X02 = dist_X.getMarginal([0, 2]).drawPDF([-4.0, -1.0], [4.0, 1.0], [256] * 2)
contour = g_X02.getDrawable(0).getImplementation()
contour.setIsFilled(True)
contour.buildDefaultLevels(50)
g_X02.setDrawable(0, contour)
g_X02.setTitle(r"$(X_0, X_2)$: iso-lines PDF")

# %%
# We draw all the conditioned distributions :math:`(X_0, X_2)|X_1 = x_1`.
cond_value_list = [coll_marg[1].computeQuantile(q)[0] for q in q_list]
graph_list = list()
graph_list.append(g_X02)

for index, cond_value in enumerate(cond_value_list):
    cond_value = cond_value_list[index]
    cond_dist = ot.PointConditionalDistribution(dist_X, cond_indices, [cond_value])
    g_cond = cond_dist.drawPDF([-4.0, -1.0], [4.0, 1.0], [256] * 2)
    contour = g_cond.getDrawable(0).getImplementation()
    contour.setIsFilled(True)
    contour.buildDefaultLevels(50)
    g_cond.setDrawable(0, contour)
    g_cond.setTitle(r"$(X_0, X_2)|X_1 = q($" + str(q_list[index]) + "): iso-lines PDF")
    g_cond.setXTitle(r"$x_0$")
    g_cond.setYTitle(r"$x_2$")
    graph_list.append(g_cond)
    print("Quantile of the cond value = ", q_list[index])
    print("Range :\n", cond_dist.getRange())

# %%
# We gather both graphs into one grid.
grid = ot.GridLayout(2, 2)
grid.setGraph(0, 0, graph_list[0])
grid.setGraph(0, 1, graph_list[1])
grid.setGraph(1, 0, graph_list[2])
grid.setGraph(1, 1, graph_list[3])
view = otv.View(grid)

# %%
# Case 4: Extreme values conditioning
# -----------------------------------
#
# Here, we show that it is possible to condition one marginal to a value which is outside its numerical range.
# Be careful that the conditioning value still must be in the theoretical range of the distribution!
#
# We consider the bivariate  distribution of :math:`(X_0, X_1)` defined by:
#
# ===========      ==================================================  ===============================
# Variable         Distribution                                        Parameter
# ===========      ==================================================  ===============================
# :math:`X_0`      :class:`~openturns.Normal` (:math:`\mu, \sigma`)    :math:`(\mu, \sigma) = (0,1)`
# :math:`X_1`      :class:`~openturns.Normal` (:math:`\mu, \sigma`)    :math:`(\mu, \sigma) = (0,1)`
# Copula           :class:`~openturns.ClaytonCopula` (:math:`\theta`)  :math:`\theta = 2`
# ===========      ==================================================  ===============================
#
# We condition the marginal :math:`X_1` to be equal to the value :math:`x_1 = -9`, which
# is outside the numerical range of its distribution: :math:`[-7.65063, +7.65063]`,
# but inside the theoretical range which is :math:`\Rset`.
#
# We draw the distribution of:
#
# .. math::
#
#    X_0 | X_1 = -9.0
#
dist_X = ot.JointDistribution([ot.Normal(), ot.Normal()], ot.ClaytonCopula(2.0))
cond_dist = ot.PointConditionalDistribution(dist_X, [1], [-9.0])

g_cond = cond_dist.drawPDF(-10.0, -8.0, 256)
g_cond.setTitle(r"$X_0|X_1 = -9$: iso-lines PDF")
g_cond.setXTitle(r"$x_0$")
g_cond.setLegendPosition("")
view = otv.View(g_cond)

# %%
# We print the updated numerical range of :math:`X_0| X_1 = -9.0`. We note that it has moved into
# :math:`[-11.76, -0.82]` which is not included in the numerical range of :math:`X_0`.
print("Range of X0|X1 = -9.0 :\n", cond_dist.getRange())


# %%
view.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# 1. Basic 1D linear function
func1 = ot.SymbolicFunction(["x1", "x2"], ["3*x1 - 2*x2 +5"])
lin1 = func1.isLinear()
lin10 = func1.isLinearlyDependent(0)
lin11 = func1.isLinearlyDependent(1)
assert lin1
assert lin10
assert lin11

# 2. Basic 2D linear function
func2 = ot.SymbolicFunction(["x1", "x2"], ["3*x1 - 2*x2 +5", "-x1 + x2"])
lin2 = func2.isLinear()
lin20 = func2.isLinearlyDependent(0)
lin21 = func2.isLinearlyDependent(1)
assert lin2
assert lin20
assert lin21

# 3. 2D non linear function
func3 = ot.SymbolicFunction(["x1", "x2"], ["3*x1 - 2*x2 +5", "-x1^2 + x2"])
lin3 = func3.isLinear()
lin30 = func3.isLinearlyDependent(0)
lin31 = func3.isLinearlyDependent(1)
assert not lin3
assert not lin30
assert lin31


# 4. 2D non linear function
func4 = ot.SymbolicFunction(["x1", "x2"], ["3*x1*x2"])
lin4 = func4.isLinear()
lin40 = func4.isLinearlyDependent(0)
lin41 = func4.isLinearlyDependent(1)
assert not lin4
assert lin40
assert lin41

# 5. Aggregated function
func5 = ot.AggregatedFunction([func2, func4])
lin5 = func5.isLinear()
lin50 = func5.isLinearlyDependent(0)
lin51 = func5.isLinearlyDependent(1)
assert not lin5
assert lin50
assert lin51

# 6. Python function


def pyFunc6(x):
    x1, x2 = x
    result = [3 * x1 - 2 * x2 + 5, -x1 + x2, 3 * x1 * x2]
    return result


def funLinearity():
    return False


def varLinearity(index):
    return True


func6 = ot.PythonFunction(
    2, 3, pyFunc6, functionLinearity=funLinearity, variablesLinearity=varLinearity
)
lin6 = func6.isLinear()
lin60 = func6.isLinearlyDependent(0)
lin61 = func6.isLinearlyDependent(1)
assert not lin6
assert lin60
assert lin61



"""
Mix/max search using optimization
=================================
"""

# %%
#
# In this example we are going to evaluate the min and max values of the output variable of interest in a domain using an optimization algorithm.

# %%
import openturns as ot


# %%
# Create the marginal distributions of the parameters
dist_E = ot.Beta(0.93, 2.27, 2.8e7, 4.8e7)
dist_F = ot.LogNormalMuSigma(30000, 9000, 15000).getDistribution()
dist_L = ot.Uniform(250, 260)
dist_I = ot.Beta(2.5, 1.5, 3.1e2, 4.5e2)
marginals = [dist_E, dist_F, dist_L, dist_I]
distribution = ot.JointDistribution(marginals)

# %%
# Define bounds
lowerBound = [marginal.computeQuantile(0.1)[0] for marginal in marginals]
upperBound = [marginal.computeQuantile(0.9)[0] for marginal in marginals]
bounds = ot.Interval(lowerBound, upperBound)

# %%
# Create the model
model = ot.SymbolicFunction(["E", "F", "L", "I"], ["F*L^3/(3*E*I)"])

# %%
# Define the problems
minProblem = ot.OptimizationProblem(model)
minProblem.setBounds(bounds)

maxProblem = ot.OptimizationProblem(model)
maxProblem.setBounds(bounds)
maxProblem.setMinimization(False)

# %%
# Create a solver
solver = ot.TNC()
solver.setStartingPoint(distribution.getMean())

# %%
# Solve the problems
solver.setProblem(minProblem)
solver.run()
minResult = solver.getResult()
print("min: y=", minResult.getOptimalValue(), "with x=", minResult.getOptimalPoint())

solver.setProblem(maxProblem)
solver.run()
maxResult = solver.getResult()
print("max: y=", maxResult.getOptimalValue(), "with x=", maxResult.getOptimalPoint())



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

grid = ot.GridLayout(2, 3)
palette = ot.Drawable.BuildDefaultPalette(10)
for j in range(grid.getNbColumns()):
    alpha = 1.0 + j
    pdf_curve = ot.WeibullMin(1.0, alpha, 0.0).drawPDF()
    cdf_curve = ot.WeibullMin(1.0, alpha, 0.0).drawCDF()
    pdf_curve.setColors([palette[j]])
    cdf_curve.setColors([palette[j]])
    legend = f"alpha={alpha}"
    pdf_curve.setLegends([legend])
    cdf_curve.setLegends([legend])
    grid.setGraph(0, j, pdf_curve)
    grid.setGraph(1, j, cdf_curve)
graph = grid

fig = plt.figure(figsize=(12, 8))
otv.View(graph, figure=fig, add_legend=True)



"""
Evaluate the mean of a random vector by simulations
===================================================
"""

# %%
# Abstract
# --------
# We introduce the :class:`~openturns.ExpectationSimulationAlgorithm` class which implements
# an incremental Monte Carlo sampling algorithm to estimate the mean of a random vector.
import openturns as ot
import openturns.viewer as otv

# %%
# We shall use this algorithm for the :ref:`Ishigami function <use-case-ishigami>` that we load from the `usecases` module :
#
from openturns.usecases import ishigami_function

im = ishigami_function.IshigamiModel()


# %%
# The Ishigami `model` and the distribution of the input variables are stored in
# the `im` object :
model = im.model
distribution = im.distribution

# %%
# We create a random vector that follows the distribution of the input variables.
inputVector = ot.RandomVector(distribution)

# %%
# The output vector is a :class:`~openturns.CompositeRandomVector`.
outputVector = ot.CompositeRandomVector(model, inputVector)

# %%
# The mean of the output vector is
print("Mean of the output random vector : %.5f" % im.expectation)


# %%
# We define the algorithm simply by calling it with the output vector :
algo = ot.ExpectationSimulationAlgorithm(outputVector)

# %%
# We can also set the algorithm parameters :
algo.setMaximumOuterSampling(80000)
algo.setBlockSize(1)
algo.setCoefficientOfVariationCriterionType("NONE")


# %%
# We are then ready to launch the algorithm and store the result.
algo.run()
result = algo.getResult()


# %%
# As usual for Monte Carlo estimation we can draw the convergence history.
graphConvergence = algo.drawExpectationConvergence()
view = otv.View(graphConvergence)


# %%
# The result obtained with the previous algorithm is an instance of the
# :class:`~openturns.ExpectationSimulationResult` class.

# %%
# The expected value of the mean is given by the `getExpectationEstimate` method :
expectation = result.getExpectationEstimate()
print("Estimated mean of the output random vector : %.5f" % expectation[0])

# %%
# The variance and standard deviation of the estimated mean are respectively given by `getVarianceEstimate` and `getStandardDeviation`:
expectationVariance = result.getVarianceEstimate()
print(
    "Variance of the estimated mean of the output random vector : %.5f"
    % expectationVariance[0]
)
standardDeviation = result.getStandardDeviation()
print("Standard deviation : %.5f" % standardDeviation[0])

# %%
# This variance and this standard deviation must not to be confused with the variance and the standard deviation of the Ishigami model!
print("Ishigami variance : %.5f" % im.variance)
print("Ishigami standard deviation : %.5f" % im.variance ** (1 / 2))


# %%
# The asymptotic confidence distribution of the output random vector mean estimate is
expectationDistribution = result.getExpectationDistribution()
print(expectationDistribution)

# %%
# Let us draw it:
graphExpectationDistribution = expectationDistribution.drawPDF()
graphExpectationDistribution.setTitle(
    "Normal asymptotic distribution of the mean estimate"
)
view = otv.View(graphExpectationDistribution)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Dimension of test
dimension = 3

# Lambda
lambdaVector = ot.Point(dimension)
lambdaVector[0] = 0.0
lambdaVector[1] = 0.5
lambdaVector[2] = 1

# Init of the function
myFunction = ot.InverseBoxCoxEvaluation(lambdaVector)
myFunction.setName("myInverseBoxCoxFunction")

# We check that for a Point with same values, the function is
# done by component
inPoint = ot.Point(dimension, 2.0)

# result point of the function
outPoint = myFunction(inPoint)
print("myFunction=", myFunction)
print(myFunction.getName(), "( ", inPoint, " ) = ", outPoint)

# Creation of a Sample
size = 10
inSample = ot.Sample(size, dimension)
for index in range(size):
    inSample[index] = ot.Point(dimension, (1.0 / (index + 1)))

# result of the function
outSample = myFunction(inSample)
print(myFunction.getName(), "( ", repr(inSample), " ) = ", repr(outSample))



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# 1D tests
distribution = ot.Dirac(0.7)
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", oneRealization)

# Test for sampling
size = 10000
oneSample = distribution.getSample(size)
print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
print("mean=", oneSample.computeMean())
sampleCovariance = oneSample.computeCovariance()[0, 0]
if abs(sampleCovariance) < 1.0e-16:
    sampleCovariance = 0.0
print("covariance=", sampleCovariance)

# Define a point
point = ot.Point(distribution.getDimension(), 0.0)
print("Point= ", point)

# Show PDF and CDF of point
PDF = distribution.computePDF(point)
print("pdf     =", PDF)
print(
    "pdf (FD)=",
    (
        distribution.computeCDF(point + ot.Point(1, 0))
        - distribution.computeCDF(point + ot.Point(1, -1))
    ),
)
CDF = distribution.computeCDF(point)
print("cdf=", CDF)

# Define a point
point = ot.Point(distribution.getSupport(distribution.getRange())[0])
print("Point= ", point)

# Show PDF and CDF of point
PDF = distribution.computePDF(point)
print("pdf     =", PDF)
print(
    "pdf (FD)=",
    (
        distribution.computeCDF(point + ot.Point(1, 0))
        - distribution.computeCDF(point + ot.Point(1, -1))
    ),
)
CDF = distribution.computeCDF(point)
print("cdf=", CDF)
CF = distribution.computeCharacteristicFunction(0.5)
print("characteristic function= (%.12g%+.12gj)" % (CF.real, CF.imag))
GF = distribution.computeGeneratingFunction(0.5 + 0.3j)
print("generating function= (%.12g%+.12gj)" % (GF.real, GF.imag))
quantile = distribution.computeQuantile(0.95)
# Test invalid quantile
try:
    q12 = distribution.computeScalarQuantile(1.2)
    raise ValueError(f"Expected a TypeError but got quantile(1.2) = {q12}")
except TypeError:
    pass
print("quantile=", quantile)
print("cdf(quantile)=", distribution.computeCDF(quantile))
print("entropy=%.6f" % distribution.computeEntropy())
mean = distribution.getMean()
print("mean=", mean)
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", standardDeviation)
skewness = distribution.getSkewness()
print("skewness=", skewness)
kurtosis = distribution.getKurtosis()
print("kurtosis=", kurtosis)
covariance = distribution.getCovariance()
print("covariance=", covariance)
parameters = distribution.getParametersCollection()
print("parameters=", parameters)
print("Standard representative=", distribution.getStandardRepresentative())

# N-D tests

dim = 4
distribution = ot.Dirac(ot.Point(dim, 2.3))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", oneRealization)

# Test for sampling
size = 10000
oneSample = distribution.getSample(size)
print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
print("mean=", oneSample.computeMean())
sampleCovariance = oneSample.computeCovariance()
for i in range(dim):
    for j in range(i + 1):
        if abs(sampleCovariance[i, j]) < 1.0e-16:
            sampleCovariance[i, j] = 0.0
print("covariance=", sampleCovariance)

# Define a point
point = ot.Point(dim, 0.0)
print("Point= ", point)

# Show PDF and CDF of point
PDF = distribution.computePDF(point)
print("pdf     =", PDF)
print(
    "pdf (FD)=",
    (
        distribution.computeCDF(point + ot.Point(dim, 0))
        - distribution.computeCDF(point + ot.Point(dim, -1))
    ),
)
CDF = distribution.computeCDF(point)
print("cdf=", CDF)

# Define a point
point = ot.Point(distribution.getSupport(distribution.getRange())[0])
print("Point= ", point)

# Show PDF and CDF of point
PDF = distribution.computePDF(point)
print("pdf     =", PDF)
print(
    "pdf (FD)=",
    (
        distribution.computeCDF(point + ot.Point(dim, 0))
        - distribution.computeCDF(point + ot.Point(dim, -1))
    ),
)
CDF = distribution.computeCDF(point)
print("cdf=", CDF)

quantile = distribution.computeQuantile(0.95)
print("quantile=", quantile)
print("cdf(quantile)=", distribution.computeCDF(quantile))
mean = distribution.getMean()
print("mean=", mean)
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", standardDeviation)
skewness = distribution.getSkewness()
print("skewness=", skewness)
kurtosis = distribution.getKurtosis()
print("kurtosis=", kurtosis)
covariance = distribution.getCovariance()
print("covariance=", covariance)
parameters = distribution.getParametersCollection()
print("parameters=", parameters)
print("Standard representative=", distribution.getStandardRepresentative())

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.skipMoments()
validation.skipCorrelation()
validation.skipConditional()  # FIXME
validation.run()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


# Simplified interfaces
# First, try 1D drawing
f = ot.SymbolicFunction("x", "sin(2*pi_*x)*exp(-x^2/2)")
print("f=", f)
graph = f.draw(-1.2, 1.2, 32)
print("graph=", graph)

# Second, try 2D drawing
f = ot.SymbolicFunction(["x", "y"], ["2.0+x-2*y+x*y-x^2-3*y^2+x*y^2"])
print("f=", f)
graph = f.draw(ot.Point(2, -10.0), ot.Point(2, 10.0), ot.Indices(2, 21))
print("graph=", graph)

# Full interfaces
f = ot.SymbolicFunction(
    ["x0", "x1", "x2"],
    [
        "x0 * sin(x1 + 2.4 * x2) - 2.0 * x1 * cos(3.2 * x0 - x2)",
        "x1 * cos(x2 + 2.4 * x1) + 2.0 * x0 * cos(3.2 * x1 - x0)",
    ],
)
centralPoint = ot.Point([1.0, -0.5, 1.5])
# First output as a function of first input around central point
graph1D = f.draw(0, 0, centralPoint, -5.0, 5.0, 32)
print("graph1D=", graph1D)
# Second output as a function of second and third inputs around central
# point
graph2D = f.draw(
    1, 2, 1, centralPoint, ot.Point(2, -5.0), ot.Point(2, 5.0), ot.Indices(2, 21)
)
print("graph2D=", graph2D)
# Cross cuts
crossCuts = f.getMarginal(0).drawCrossCuts(
    centralPoint, ot.Point(3, -5.0), ot.Point(3, 5.0), ot.Indices(3, 10)
)
print("crossCuts=", crossCuts)
crossCuts = f.getMarginal(1).drawCrossCuts(
    centralPoint,
    ot.Point(3, -5.0),
    ot.Point(3, 5.0),
    ot.Indices(3, 3),
    True,
    False,
    -5.0,
    3.0,
)
print("crossCuts=", crossCuts)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# We create a numerical point of dimension 4
point = ot.Point([101.0, 102.0, 103.0, 104.0])

print("point = ", repr(point))

# We create a 'constant' RandomVector from the Point
vect = ot.RandomVector(ot.ConstantRandomVector(point))
print("vect=", vect)

# Check standard methods of class RandomVector
print("vect dimension=", vect.getDimension())
print("vect realization (first )=", repr(vect.getRealization()))
print("vect realization (second)=", repr(vect.getRealization()))
print("vect realization (third )=", repr(vect.getRealization()))
print("vect sample =", repr(vect.getSample(5)))
